\title{The CESAW dataset: a conversation}
\author{Derek M. Jones\\Knowledge Software\\derek@knosof.co.uk
\and William R. Nichols\\Carnegie Mellon University\\wrn@andrew.cmu.edu}
\maketitle

<<label=read-data,include=FALSE,echo=FALSE>>=

#  6 Jun 21
CRAN_dir=Sys.getenv("CRAN_dir")
if (CRAN_dir == "")
   CRAN_dir ="http://cran.us.r-project.org"

if (!require(bizdays)   ) install.packages('bizdays'   , repos = CRAN_dir)
if (!require(gamlss)    ) install.packages('gamlss'    , repos = CRAN_dir)
if (!require(gamlss.tr) ) install.packages('gamlss.tr' , repos = CRAN_dir)
if (!require(igraph)    ) install.packages('igraph'    , repos = CRAN_dir)
if (!require(lubridate) ) install.packages('lubridate' , repos = CRAN_dir)
if (!require(plyr)      ) install.packages('plyr'      , repos = CRAN_dir)
if (!require(seriation) ) install.packages('seriation' , repos = CRAN_dir)
if (!require(gnm)       ) install.packages('gnm'       , repos = CRAN_dir)
if (!require(grid)      ) install.packages('grid'       , repos = CRAN_dir)
#if (!require(tinytex)   ) install.packages('tinytex'   , repos = CRAN_dir)

library(bizdays)
library(gamlss)
library(gamlss.tr)
library(igraph)
library(lubridate)
library(plyr)
library(seriation)
library(gnm)
library(grid)
#library(tinytex)


CESAW_dir=Sys.getenv("CESAW_dir")
if (CESAW_dir == "")
   CESAW_dir=getwd()

# CESAW_dir=paste0(CESAW_dir, "/")

setwd(CESAW_dir)

#setwd("D:/Work/@Projects/DJ/SEMPR_3_08")

source("ESEUR_config.r")


# library("quantreg")

par(cex.axis=1.3)
par(cex.lab=1.5)

# Construction phases
c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

tf=read.csv("data/CESAW_task_fact.csv.xz", as.is=TRUE)

tf$start_date=as.Date(tf$task_actual_start_date, format="%m/%d/%Y")
tf$end_date=as.Date(tf$task_actual_complete_date, format="%m/%d/%Y")

tf$is_construct=(tf$phase_short_name %in% c_phase)

timf=read.csv("data/CESAW_time_fact.csv.xz", as.is=TRUE)

timf$start_date=as.Date(timf$time_log_start_date, format="%m/%d/%Y")
timf$end_date=as.Date(timf$time_log_end_date, format="%m/%d/%Y")

timf$organization[timf$organization == ""]="B" # Clean data

timf$is_construct=(timf$phase_short_name %in% c_phase)

# Projects with the most rows
p95=subset(tf, project_key == 95)   # 2nd
p614=subset(tf, project_key == 614) # 4th
p615=subset(tf, project_key == 615) # 1st
p617=subset(tf, project_key == 617) # 3rd

# Remove outliers
p614=subset(p614, start_date > as.Date("2014-06-18"))
p615=subset(p615, start_date > as.Date("2010-01-01"))

proj_range=c(0, 1+length(unique(tf$project_key)))

# Create a US calendar without any holidays!
create.calendar("US", weekdays=c("saturday", "sunday"))

# # Some projects are listed in Time facts but not Task facts:
# taskp=unique(task$project_key)
# timp=unique(timf$project_key)
# taskp[!(taskp %in% timp)]
# # [1] 100  57  73  80  81  85
# taskp[!(timp %in% taskp)]
# # integer(0)
@


<<label=proj-task-freq,include=FALSE,echo=FALSE>>=
plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

plot(sort(count(tf$project_key)$freq), log="y", col=point_col,
	xaxs="i",
	xlim=proj_range,
	xlab="Project", ylab="Tasks (sorted)\n")

p_p=ddply(tf, .(project_key), function(df) length(unique(df$person_key)))

plot(sort(p_p$V1), log="y", col=point_col,
	xaxs="i", yaxs="i",
	xlim=proj_range, ylim=c(1, 50),
	xlab="Project", ylab="People (sorted)\n")
@
\begin{figure}
\begin{center}
<<label=proj-task,fig=TRUE,echo=FALSE>>=
<<proj-task-freq>>
@
\end{center}
\caption{Left: Number of tasks (in sorted order) recorded for each project; right: Number of people (in sorted order) who worked on each project.}
\label{proj-task:fig}
\end{figure}

<<label=proj-WBS-tree,include=FALSE,echo=FALSE>>=
plot_wide()
# Look for patterns or structure in the wbs_element tree

pal_col=rainbow(7)

p615$syear=as.numeric(substr(p615$start_date, 3, 4))

wtree=read.csv("data/wbs_parent.csv", as.is=TRUE)

# Pick a project
wp615=subset(wtree, project_key == 615)

uwp615=unique(wp615)
uw_from=unique(uwp615$parent_wbs_element_key)
uw_to=unique(uwp615$wbs_element_key)

uwp615$color=pal_col[2] # Non-leaf color
v_color=rep(pal_col[1], nrow(uwp615))
# leaf nodes don't have a to-node in the list of from-nodes
# uwp615$color[!(uwp615$wbs_element_key %in% uw_from)]=pal_col[3]

# Pick one of each wbs_element_key, to get its start_year
u_p615=subset(p615, !duplicated(wbs_element_key))

uwp615$leaf_p=(uwp615$wbs_element_key %in% u_p615$wbs_element_key)
uwp615$color[uwp615$leaf_p]=pal_col[mapvalues(uwp615$wbs_element_key[uwp615$leaf_p], u_p615$wbs_element_key, u_p615$syear)-9+2]
v_color[uwp615$leaf_p]="black"

ant_g=graph.data.frame(data.frame(uwp615$parent_wbs_element_key, uwp615$wbs_element_key, color=uwp615$color),
                        directed=TRUE,
                        vertices=data.frame(c(uwp615$wbs_element_key, "NA"), color=c(v_color, 0)))

V(ant_g)$label=NA
V(ant_g)$size=0.3
# V(ant_g)$color=pal_col[2]
E(ant_g)$arrow.size=0.2
# E(ant_g)$color=pal_col[2]

plot(ant_g, layout=layout_as_tree,
        vertex.frame.color=V(ant_g)$color)

legend(x="topleft", legend=c("Leaf", "Non-leaf", paste0("20", 10:14)), bty="n", fill=c("black", pal_col), cex=1.2)

@
\begin{figure}
\begin{center}
<<label=WBS-tree,fig=TRUE,echo=FALSE>>=
<<proj-WBS-tree>>
@
\end{center}
\caption{The Work Breakdown Structure tree of the 961 wbs elements in project 615; leaf WBS elements (in black, contain a sequence of tasks), non-leaf WBS elements reference other WBS elements (in red; or terminate with no corresponding tasks), other colored edges denote year in which associated leaf tasks were first started.}
\label{WBS-tree:fig}
\end{figure}

<<label=proj-WBS-duration,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

wbs_totals=function(df)
{
return(data.frame(
                  num_tasks=nrow(df),
                  num_persons=length(unique(df$person_key)),
                  persons=paste0(sort(unique(df$person_key)), collapse=","),
                  num_phases=length(unique(df$phase_short_name)),
                  phases=paste0(sort(unique(df$phase_short_name)), collapse=","),
                  # phases=paste0(unique(df$phase_short_name), collapse=","),
                  est_total=sum(df$task_plan_time_minutes),
                  act_total=sum(df$task_actual_time_minutes),
                  total_days=sum(df$task_actual_time_minutes)/(60*7),
                  start_date=min(df$start_date, na.rm=TRUE),
                  end_date=max(df$end_date, na.rm=TRUE),
                  stringsAsFactors=FALSE
                  ))
}

p_dates=ddply(tf, .(project_key), function(df) range(df$start_date, na.rm=TRUE))
plot(sort(p_dates$V2-p_dates$V1)+1, log="y", col=point_col,
	xaxs="i", yaxs="i",
	xlim=proj_range,
	xlab="Project", ylab="Duration (sorted by days)\n")

tf_wbs=ddply(tf, .(wbs_element_key), wbs_totals)
p615_wbs=ddply(p615, .(wbs_element_key), wbs_totals)

wbs_dur=bizdays::bizdays(tf_wbs$start_date, tf_wbs$end_date, "US")
wbs_freq=count(wbs_dur)
wbs_freq$x=wbs_freq$x+1 # start/end on same day is counted as zero days

plot(wbs_freq, log="xy", col=pal_col[2],
        xlim=c(1, 500),
        xlab="Working days", ylab="WBS\n")

wbs_dur=bizdays::bizdays(p615_wbs$start_date, p615_wbs$end_date, "US")
wbs_freq$x=wbs_freq$x+1
wbs_freq=count(wbs_dur)

points(wbs_freq, col=pal_col[1])

legend(x="topright", legend=c("All projects", "Project 615"), bty="n", fill=pal_col[2:1], cex=1.3)

@
\begin{figure}
\begin{center}
<<label=proj_WBS-d,fig=TRUE,echo=FALSE>>=
<<proj-WBS-duration>>
@
\end{center}
\caption{Left: Duration of each project, sorted by days; right: Number of WBS whose implemented occurs over a given number of working days, for all projects and project 615.}
\label{proj_WBS-d:fig}
\end{figure}


<<label=proj-WBS-freq,include=FALSE,echo=FALSE>>=
plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

p_wbs=ddply(tf, .(project_key), function(df) length(unique(df$wbs_element_key)))

plot(sort(p_wbs$V1), log="y", col=point_col,
        xaxs="i", yaxs="i", 
        xlim=proj_range, ylim=c(5, 650),
        xlab="Project", ylab="WBS elements (sorted)\n")

wbs_freq=count(tf$wbs_element_key)

plot(sort(wbs_freq$freq), log="y", col=point_col,
        xaxs="i", yaxs="i",
        ylim=c(0.9, 500),
        xlab="WBS element", ylab="Tasks (sorted)\n")

# head(table(wbs_freq$freq))/nrow(wbs_freq)
# head(cumsum(table(wbs_freq$freq)))/nrow(wbs_freq)

@
\begin{figure}
\begin{center}
<<label=proj-WBS,fig=TRUE,echo=FALSE>>=
<<proj-WBS-freq>>
@
\end{center}
\caption{Left: Number of WBS (in sorted order) contained in each project; right: Number of tasks (in sorted order) contained in each WBS for all projects.}
\label{proj-WBS:fig}
\end{figure}


<<label=est_act_fit,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(mar=MAR_default+c(0.0, 1.2, 0, 0)) # leave more room for label

par(cex.axis=1.3)
par(cex.lab=1.5)

fit_est_act=function(df)
{
df_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes),
                        subset=(task_plan_time_minutes!=0), data=df)

return(coef(df_mod))
}


pal_col=rainbow(2)

# Reduce image load time by using a sample
tf_sub=sample(1:nrow(tf), nrow(tf)/10)

plot(tf$task_actual_time_minutes[tf_sub], tf$task_plan_time_minutes[tf_sub],
		log="xy", col=pal_col[2],
		xlab="Estimated time (minutes)", ylab="Actual time (minutes)\n\n")

# tf_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)+factor(person_key)+factor(team_key)+phase_key,
tf_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes),
                        subset=(task_plan_time_minutes!=0), data=tf)
# summary(tf_mod)
x_vals=seq(1, 1e4, by=10)
pred=predict(tf_mod, newdata=data.frame(task_plan_time_minutes=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])

fit_mods=ddply(tf, .(project_key), fit_est_act)

plot(sort(fit_mods$"log(task_plan_time_minutes)"), col=point_col,
	xaxs="i",
	xlim=proj_range,
	xlab="Project", ylab="Exponent (sorted)\n")
@
\begin{figure}
\begin{center}
<<label=est_act,fig=TRUE,echo=FALSE>>=
<<est_act_fit>>
@
\end{center}
\caption{Left: Estimated task work time (in minutes) against actual task work time for the 61,817 tasks in all projects, with fitted regression model of the form $Actual \propto Estimated^{0.85}$; right: Exponent of models fitted (in sorted order) to every project.}
\label{est_act:fig}
\end{figure}

\begin{verbatim}
<<label=task-freq,results=tex,echo=FALSE>>=

t=count(p615$phase_short_name)
t=t[order(t$freq, decreasing=TRUE), ]
names(t)=c("Task", "Occurrences")
row.names(t)=NULL

# Print as two columns
rlen=round(nrow(t)/2+0.1)
print(cbind(t[1:rlen,], t[(rlen+1):nrow(t),]), row.names=FALSE)
@
\end{verbatim}

<<label=task-cont_days,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

# How many tasks are continued for N days?
continuation_days=function(df)
{
proj=subset(df, (start_date <= end_date) &
                        !(is.na(start_date) | is.na(end_date)))

# There is a bizdays in forecast!
task_dur=bizdays::bizdays(proj$start_date, proj$end_date, "US")
task_freq=count(task_dur)
plot(task_freq, log="xy", col=pal_col[2],
        xlim=c(1, 500), ylim=c(1, 6500),
        xlab="Continuation working days", ylab="Tasks\n")

points(0.9, task_freq$freq[task_freq$x == 0], pch="C", cex=1.2, col="red")
tf_mod=glm(log(freq) ~ log(x), data=task_freq, subset=((x > 0) & (x < 10)))
# print(summary(tf_mod))
pred=predict(tf_mod)
lines(exp(pred), col=pal_col[1])

x_vals=10:59
tf_mod=glm(log(freq) ~ log(x), data=task_freq, subset=((x > 9) & (x < 60)))
# print(summary(tf_mod))
pred=predict(tf_mod, newdata=data.frame(x=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])
}

continuation_days(p615)
# continuation_days(p614)
# continuation_days(p617)
# continuation_days(p95)

# Interval between two tasks sessions occurring on the same day
# Convert HH:MM to minutes
get_day_time=function(task_date)
{
s_datetime=unlist(strsplit(task_date, " "))
s_HM=strsplit(s_datetime[seq(2, length(s_datetime), by=2)], ":")
s_mins=sapply(s_HM, function(X) return(as.numeric(X[1])*60+as.numeric(X[2])))
return(s_mins)
}

# Task interrupt time, as-in interval between sessions

# space separates date from time
se_tf=subset(timf, grepl(" ", time_log_start_date) &
                        grepl(" ", time_log_end_date))

se_tf$start_time=get_day_time(se_tf$time_log_start_date)
se_tf$end_time=get_day_time(se_tf$time_log_end_date)

# Summary information about a person
person_totals=function(df)
{
return(data.frame(person_key=df$person_key[1],
                        num_tasks=nrow(df),
                        num_wbs=length(unique(df$wbs_element_key)),
                        num_phases=length(unique(df$phase_short_name)),
                        unq_est=length(unique(df$task_plan_time_minutes))/nrow(df),
                        unq_act=length(unique(df$task_actual_time_minutes))/nrow(df),
                        act_total=sum(df$task_actual_time_minutes),
                        total_days=sum(df$task_actual_time_minutes)/(60*7),
                        stringsAsFactors=FALSE
                        ))
}


# What is the diversity of a person's estimates and actuals?

person_info=ddply(p615, .(person_key), person_totals)
p10=subset(person_info, num_tasks > 20)

est_order=order(p10$unq_est)

plot(p10$unq_act[est_order], col=pal_col[1],
        yaxs="i",
        ylim=c(0.2, 1),
        xlab="Person", ylab="Unique values (fraction)\n")
points(p10$unq_est[est_order], col=pal_col[2])

legend(x="bottomright", legend=c("Actual", "Estimated"), bty="n", fill=pal_col, cex=1.3)

@
\begin{figure}
\begin{center}
<<label=cont_days,fig=TRUE,echo=FALSE>>=
<<task-cont_days>>
@
\end{center}
\caption{Both plots are for project 615.  Left: Number of tasks that continue to be worked for a given number of working days after the first day, red C denotes all tasks completed on the day they were started; lines are fitted power laws with exponents of -1 and -1.6; right: Fraction of task estimates and actuals that are unique to each person who implemented at least 20 tasks, with people sorted by estimate fraction.}
\label{cont_days:fig}
\end{figure}


<<label=zt-nb_fit,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

# How many tasks does a person have in-flight on any day?
tasks_per_day=function(df)
{
p_days=day_zero # Start with all project days zero

sum_days=function(df)
   {
# The end date may be later, assume stop button hit the next day
   day_range=1+as.integer(df$start_date-p_start_date)
   p_days[day_range]<<-p_days[day_range]+1
   return(NULL)
   }

a_ply(df, 1, sum_days)
return(p_days)
}

task_times=function(df)
{
task_inf=data.frame(num_sess=nrow(df),
                        day_diff=0,
                        person_key=df$person_key[1],
                        plan_item_key=df$plan_item_key[1],
                        wbs_element_key=df$wbs_element_key[1],
                        phase_short_name=df$phase_short_name[1],
                        is_construct=df$is_construct[1],
                        time_log_start_date=min(df$time_log_start_date),
                        total_minutes=sum(df$time_log_delta_minutes),
                        interrupt_minutes=sum(df$time_log_interrupt_minutes),
			stringsAsFactors=FALSE
                        )
task_inf$day_diff=as.integer(max(df$start_date)-min(df$start_date))

return(task_inf)
}

merge_plan_time=function(df)
{
tptm=data.frame(person_key=df$person_key,
                plan_item_key=df$plan_item_key,
		task_actual_time_minutes=df$task_actual_time_minutes,
                task_plan_time_minutes=df$task_plan_time_minutes)

t=merge(tasks, tptm)

return(t)
}

tproj=subset(timf, project_key == 615)
# (person_key, plan_item_key) provides a unique combination for a task
tasks=ddply(tproj, .(person_key, plan_item_key), task_times)
pt=merge_plan_time(p615)

p_start_date=min(tproj$start_date)
p_num_days=1+as.integer(max(tproj$end_date)-p_start_date)
day_zero=rep(0, p_num_days)

pt_per_day=daply(tproj, .(person_key), tasks_per_day)

# Fit a zero-truncated negative binomial distribution

plot(count(as.vector(pt_per_day)), log="y", col=pal_col[2],
        xlim=c(1, 25), ylim=c(1, 2100),
        xlab="Tasks worked on during day", ylab="Days\n")

# Need to explicitly specify where the distribution is to be truncated
gen.trun(par=0, family=NBII, print=FALSE)

day_list=as.vector(pt_per_day)
day_list=day_list[(day_list > 0) & (day_list < 21)]

# Fit a zero-truncated, type II, negative binomial distribution
g_NBIItr=gamlss(day_list ~ 1, family=NBIItr, trace=FALSE)

# print(summary(g_NBIItr))

NBII_mu=exp(coef(g_NBIItr, "mu"))
NBII_sigma=exp(coef(g_NBIItr, "sigma"))

# print(c(NBII_mu, NBII_sigma))

lines(dNBIItr(1:20, mu=NBII_mu, sigma=NBII_sigma)*length(day_list), col=pal_col[1])

# AIC=as.numeric(AIC(g_NBIItr)
# log_likelihood=as.numeric(logLik(g_NBIItr))

total_per_day=colSums(pt_per_day)

t=total_per_day
t[t == 0]=0.7
plot(t, log="y", col=point_col,
        xaxs="i", yaxs="i",
        xlim=c(500, 2230),
        xlab="Days since project started", ylab="Tasks worked on each day\n")

@
\begin{figure}
\begin{center}
<<label=zt-nb,fig=TRUE,echo=FALSE>>=
<<zt-nb_fit>>
@
\end{center}
\caption{Left: Number of days on which a given number of tasks were worked on during a day's work, for project 615, red line is a fitted zero-truncated Negative binomial distribution, right: Total number of tasks worked on each day of project 615.}
\label{zt-nb:fig}
\end{figure}


<<label=weekly-tasks_delim,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

wd_ind=seq(1, length(total_per_day), by=7)

pt_per_wday=sapply(0:6, function(X) sum(total_per_day[X+wd_ind], na.rm=TRUE))

plot(pt_per_wday, type="b", col=point_col,
	xaxs="i", yaxs="i",
	xaxt="n",
	xlim=c(0.9, 7.1), ylim=c(0, 12000),
	xlab="", ylab="Tasks\n")
axis(1, at=1:7, label=c("Monday", "", "Wednesday", "", "Friday", "", "Sunday"))

acf(total_per_day, col=point_col,
	xlab="Lag (days)", ylab="ACF\n")

@
\begin{figure}
\begin{center}
<<label=weekly-tasks,fig=TRUE,echo=FALSE>>=
<<weekly-tasks_delim>>
@
\end{center}
\caption{Left: Total number of tasks worked on during a given day of the week, for project 615; right: Autocorrelation of the tasks worked on per day, for project 615.} 
\label{weekly-tasks:fig}
\end{figure}


<<label=wbs-people_project,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

project_totals=function(df)
{
if (nrow(df) < 100)
   return(NULL)
return(data.frame(num_people=length(unique(df$person_key)),
			num_tasks=nrow(df),
			num_wbs=length(unique(df$wbs_element_key)),
			num_phases=length(unique(df$phase_short_name)),
			act_total=sum(df$task_actual_time_minutes),
			stringsAsFactors=FALSE
			))
}

project_info=ddply(tf, .(project_key), project_totals)

# Analysis of Work Break Down entries

proj_min_people=13

pal_col=rainbow(length(which(project_info$num_people > proj_min_people)))

# How many people work on a wbs?
project_wbs=function(df)
{
wbs_people=ddply(df, .(wbs_element_key),
                        function(df) length(unique(df$person_key)))
total_people=count(wbs_people$V1)
if (length(unique(df$person_key)) > proj_min_people)
   {
   col_count <<- col_count+1
   lines(total_people, col=pal_col[col_count])
   }

return(total_people)
}

plot(0, type="n",
        xaxs="i", yaxs="i",
        xlim=c(1, 6), ylim=c(0, 500),
        xlab="People working on a WBS", ylab="WBS\n")
col_count=0

proj_wbs=ddply(tf, .(project_key), project_wbs)

# How many tasks per wbs?
tasks_wbs=function(df)
{
wbs_tasks=ddply(df, .(wbs_element_key),
                        function(df) nrow(df))
total_tasks=count(wbs_tasks$V1)
if (length(unique(df$person_key)) > proj_min_people)
   {
   col_count <<- col_count+1
   lines(total_tasks, col=pal_col[col_count])
   }

return(total_tasks)
}

plot(0, type="n",
        xaxs="i", yaxs="i",
        xlim=c(1, 20), ylim=c(0, 150),
        xlab="Tasks in a WBS", ylab="WBS\n")
col_count=0

proj_wbs=ddply(tf, .(project_key), tasks_wbs)

@
\begin{figure}
\begin{center}
<<label=wbs-people,fig=TRUE,echo=FALSE>>=
<<wbs-people_project>>
@
\end{center}
\caption{Left: Number of WBS elements involving a given number of people, broken down by project (colored lines), right: Number of tasks per WBS by project (colored lines).}
\label{wbs-people:fig}
\end{figure}

<<label=wbs-people-seriation,include=FALSE,echo=FALSE>>=
# Build graph of people who worked together on a project's wbs implementation

library("grid")  # needed for gpar
library("seriation")

pal_col=rainbow(7)

# Count the number of time each person works with another
# person on a wbs.
wbs_person_links=function(df)
{
u_person=unique(df$s_person)
num_persons=length(u_person)
if (num_persons == 1)
   return(NULL)

num_tasks=nrow(df)
act_total=sum(df$task_actual_time_minutes)

# Generate all pairs of people; each column is a pair
pairs=combn(u_person, 2)

t=sapply(1:dim(pairs)[2], function (X)
                                {
                                from_to=cbind(pairs[1, X], pairs[2, X])
                                person_mat[from_to] <<- person_mat[from_to]+1
                                from_to=cbind(pairs[2, X], pairs[1, X])
                                person_mat[from_to] <<- person_mat[from_to]+1
                                })
# return(NULL)
}


project_people_rel=function(proj, p_num)
{
u_proj_people=unique(proj$person_key)
total_people=length(u_proj_people)
person_mat <<- matrix(nrow=total_people, ncol=total_people, data=0)

proj$s_person=mapvalues(proj$person_key, u_proj_people, 1:length(u_proj_people))

d_ply(proj, .(wbs_element_key), wbs_person_links)

person_dist=dist(1/(person_mat+1e-5))
pser=seriate(person_dist)

pimage(person_dist, pser, key=FALSE, cex=1.0, col=rev(pal_col),
        newpage=FALSE, # needed to display multiple images within plot
        gp=gpar(cex=0.8), # needs library(grid)
        main=p_num,
        xlab="Person", ylab="Person")

return(NULL)
}

par(mar=MAR_default-c(2.7, 3.9, 0.7, 0.7))


grid.newpage()
pushViewport(viewport(layout=grid.layout(nrow=2, ncol=2)))

pushViewport(viewport(layout.pos.row=1, layout.pos.col=1))
dummy=project_people_rel(p614, "614")
popViewport()
pushViewport(viewport(layout.pos.row=1, layout.pos.col=2))
dummy=project_people_rel(p615, "615")
popViewport()
pushViewport(viewport(layout.pos.row=2, layout.pos.col=1))
dummy=project_people_rel(p617, "617")
popViewport()
pushViewport(viewport(layout.pos.row=2, layout.pos.col=2))
dummy=project_people_rel(p95, "95")
popViewport(2)

@
\begin{figure}
\begin{center}
<<label=wbs-seriation,fig=TRUE,echo=FALSE>>=
<<wbs-people-seriation>>
@
\end{center}
\caption{Clustering of pairs of people based on number of times they worked together on a WBS, for projects 614, 615, 617 and 95.}
\label{wbs-seriation:fig}
\end{figure}


<<label=task-person_timeline,include=FALSE,echo=FALSE>>=

plot_layout(2, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

# Timeline of each person's task involvement
person_timeline=function(p_df, p_num)
{
task_points=function(df)
   {
   points(df$start_date, df$person_ord, col=pal_col[df$person_ord[1]])
   }


p_unq=unique(p_df$person_key[order(p_df$start_date)])
pal_col=rainbow(length(p_unq))
p_df$person_ord=mapvalues(p_df$person_key, p_unq, 1:length(p_unq))

plot(p_df$start_date[1], 0, type="n",
	xaxs="i", yaxs="i",
	xlim=range(p_df$start_date, na.rm=TRUE), ylim=c(0.8, 0.2+length(p_unq)),
	xlab="Date", ylab="Person id\n")

text(p_df$start_date[1]+5, max(p_df$person_ord), p_num, pos=1, cex=1.2)

p_tasks=d_ply(p_df, .(person_ord), task_points)
}

person_timeline(p614, 614)
person_timeline(p615, 615)
person_timeline(p617, 617)
person_timeline(p95, 95)
@
\begin{figure}
\begin{center}
<<label=person_timeline,fig=TRUE,echo=FALSE>>=
<<task-person_timeline>>
@
\end{center}
\caption{Date on which each project member started working on a task, for projects 614, 615, 617 and 95, people sorted by date of first task.}
\label{person_timeline:fig}
\end{figure}

The values appearing in the \texttt{person\_key} column, denoting
distinct people working on a project, vary across projects, i.e., it
is not possible to match up the same person working on different
projects.


<<label=project-person_percent,include=FALSE,echo=FALSE>>=
# Time spent, per week, by a person on a project

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

person_weektime=function(p_df, col_str)
{
weektime=function(df)
   {
   wtime=ddply(df, .(person_ord), function(df) sum(df$task_actual_time_minutes, na.rm=TRUE))
   return(wtime)
   }

week_percent=function(df)
   {
   total_weeks=max(df$week_num, na.rm=TRUE)-min(df$week_num, na.rm=TRUE)+1
   total_mins=sum(df$V1, na.rm=TRUE)
   max_mins_week=max(df$V1, na.rm=TRUE)
   return(data.frame(total_weeks, total_mins, max_mins_week)) 
   }

p_unq=unique(p_df$person_key[order(p_df$start_date)])
pal_col=rainbow(length(p_unq))
p_df$person_ord=mapvalues(p_df$person_key, p_unq, 1:length(p_unq))

start_year=year(min(p_df$start_date, na.rm=TRUE))
p_df$week_num=(year(p_df$start_date)-start_year)*52+week(p_df$start_date)

p_wtime=ddply(p_df, .(week_num), weektime)

# plot(1, type="n", log="y",
# 	xlim=range(p_wtime$week_num, na.rm=TRUE), ylim=range(p_wtime$V1, na.rm=TRUE),
# 	xlab="Week", ylab="Total minutes")
pweek=ddply(p_wtime, .(person_ord), week_percent)
pweek$mins_per_week=pweek$total_mins/pweek$total_weeks
pweek=pweek[order(pweek$mins_per_week), ]
lines(pweek$mins_per_week/60, col=col_str)
# lines(pweek$max_mins_week/60, lty=2, col=col_str)

}

pal_col=rainbow(4)

plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(1, 50), ylim=c(1, 20),
	xlab="Person id", ylab="Hours per week\n")

person_weektime(p614, pal_col[1])
person_weektime(p615, pal_col[2])
person_weektime(p617, pal_col[3])
person_weektime(p95, pal_col[4])

legend(x="bottomright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)

# Number of distinct people starting a task in each week

person_perweek=function(p_df, col_str)
{
start_year=year(min(p_df$start_date, na.rm=TRUE))
p_df$week_num=(year(p_df$start_date)-start_year)*52+week(p_df$start_date)

p_perweek=ddply(p_df, .(week_num), function(df) length(unique(df$person_key)))

lines(p_perweek$V1, col=col_str)
}

pal_col=rainbow(4)

plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(1, 230), ylim=c(0, 31),
	xlab="Week", ylab="People\n")

person_perweek(p614, pal_col[1])
person_perweek(p615, pal_col[2])
person_perweek(p617, pal_col[3])
person_perweek(p95, pal_col[4])

legend(x="topright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)

@
\begin{figure}
\begin{center}
<<label=person_percent,fig=TRUE,echo=FALSE>>=
<<project-person_percent>>
@
\end{center}
\caption{Left: Average hours per week each person spent working on projects 614, 615, 617 and 95, people sorted by hours per week worked; right: Number of different people starting work on a task during the week.}
\label{person_percent:fig}
\end{figure}


<<label=task-num_ests,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

# Patterns in estimate values
# p95 # p614 # p615 # p617

total_est=count(p615$task_plan_time_minutes)
plot(total_est, log="y", col=pal_col[2],
	xaxs="i",
        xlim=c(1, 480), ylim=c(12, 2500),
        xlab="Estimate (minutes)", ylab="Tasks\n")

# subset(total_est, freq > 160)

text(12, 450, "12", col=pal_col[1], cex=1.2)
text(30, 2600, "30", col=pal_col[1], cex=1.2)
text(43, 320, "43", col=pal_col[1], cex=1.2)
text(60, 720, "60", col=pal_col[1], cex=1.2)
# text(90, 220, "12", col=pal_col[1], cex=1.2)
text(97, 280, "97", col=pal_col[1], cex=1.2)
text(122, 370, "122", col=pal_col[1], cex=1.2)
text(180, 280, "180", col=pal_col[1], cex=1.2)
# text(240,  90, "12", col=pal_col[1], cex=1.2)
text(275, 290, "275", col=pal_col[1], cex=1.2)
text(383, 200, "383", col=pal_col[1], cex=1.2)

pal_col=rainbow(3)

io=subset(p615, phase_short_name == "Inspect - Others")

x_vals=1:500 

plot(io$task_plan_time_minutes, io$task_actual_time_minutes, log="xy", col=pal_col[2],
        xlab="Estimate", ylab="Actual\n")
lines(x_vals, x_vals, col=pal_col[3]) 

io_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes), data=io)
# summary(io_mod)
pred=predict(io_mod, newdata=data.frame(task_plan_time_minutes=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])

@
\begin{figure}
\begin{center}
<<label=num_ests,fig=TRUE,echo=FALSE>>=
<<task-num_ests>>
@
\end{center}
\caption{left: Number of tasks estimated to take a given number of minutes, for project 615, with some peak values highlighted (red numbers); right: Estimates and actuals for \textsf{Inspect - Others} tasks in project 615, with fitted regression line (red) and $\mathit{actual}=\mathit{estimate}$ line (blue).}
\label{num_ests:fig}
\end{figure}


<<label=person-insp_est,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)
# plot_wide()

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

# Fit models to the inspection estimates

inspect_est=function(ins_str, proj_df)
{
ins_df=subset(proj_df, phase_short_name == ins_str)
ins_df=subset(ins_df, (task_actual_time_minutes > 0) &
                        (task_plan_time_minutes >0))

if (nrow(ins_df) < 2)
   return(NULL)
# Require that a person make some minimum number of estimates
pk=count(ins_df$person_key)
many_pk=subset(pk, freq > 4)
ins_df=subset(ins_df, person_key %in% many_pk$x)

# print(c(ins_str, nrow(ins_df)))

ins_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
                                                +num_sess
                                                +p_ID,
                                                , data=ins_df)
# print(summary(ins_mod))

return(ins_mod)
}

# Return the person_key coefficients
pe_coef=function(p_mod, num_uper)
{
# we don't want the first two, three with num_sess
c_vals=exp(coef(p_mod))[-(1:3)]
pid=as.numeric(sub("p_ID", "", names(c_vals))) # get the numeric person ID

# print(pid)
t_coef=rep(NA, num_uper)
t_coef[pid]=c_vals

return(t_coef)
}


plot_coef=function(proj_df, task_str, kind_phase)
{
# To simplify model fitting map person_key to a small integer string.
uper=unique(proj_df$person_key)
proj_df$p_ID=as.character(mapvalues(proj_df$person_key, uper, 1:length(uper)))

# The class returned by glm is a list
task_mod=alply(task_str, 1, inspect_est, proj_df)

task_mod=compact(task_mod) # Remove NULL entries from list

# Get the p_ID coefficients from each fitted model
per_coef=laply(task_mod, pe_coef, length(uper))

plot(0.1, type="n", log="y",
	xaxs="i", yaxs="i",
	xlim=c(1, length(uper)+0.5), ylim=c(0.15, 4),
	xlab="Person id", ylab="Coefficient\n")
dummy=sapply(1:nrow(per_coef),
				function(X)
				{
				points(per_coef[X, ], col=pal_col[1])
				})

dummy=sapply(1:ncol(per_coef), function(X)
				{
				lines(c(X, X), range(per_coef[ , X], na.rm=TRUE), col=pal_col[2])
				})

lines(c(1, 50), c(1, 1), col="grey")
text(ncol(per_coef)/2, 3.6, kind_phase, cex=1.4)
}

# c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

pal_col=rainbow(2)

# pt needed for num_sess

# List of Inspect tasks
t=unique(pt$phase_short_name)
inspect_str=t[grep("Inspect", t)]

plot_coef(pt, inspect_str, "Inspection")
plot_coef(pt, c_phase, "Creation")

@
\begin{figure}
\begin{center}
<<label=person-insp,fig=TRUE,echo=FALSE>>=
<<person-insp_est>>
@
\end{center}
\caption{Individual multiplicative factor, for each person who made at least five estimates, combined into equation~\ref{act-est-mod}; below grey lines shows an overestimation trend, and above grey line an underestimation trend, for the seven possible inspection tasks (left) and seven possible creation tasks (right), for project 615.}
\label{person-insp:fig}
\end{figure}


<<label=task-actual_freq,include=FALSE,echo=FALSE>>=
# Elapsed time for tasks completed within same day

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

unloadNamespace("igraph") # defines a gnm function

# Fit biexponential to task actual time

fit_task_act=function(df)
{
# tatm=count(df$task_plan_time_minutes)
tatm=count(df$task_actual_time_minutes)
plot(tatm$x, tatm$freq, log="y", col=pal_col[3],
	xaxs="i",
        xlim=c(0, 1000),
        xlab="Work time (minutes)", ylab="Tasks\n")

# Find 'spike' values
tatm$spl=c(head(tatm$freq, -1)/pmax(tail(tatm$freq, -1), 3), NA)
tatm$spr=c(NA, tail(tatm$freq, -1)/pmax(head(tatm$freq, -1), 3))
# tatm$dspl=c(NA, diff(tatm$spl))
# tatm$tsp=0.8+0.5*log(tatm$x)
# tatm$x[which(tatm$sp > 1+0.5*log(tatm$x))]

spike_ind=which((tatm$spl > 0.5+0.3*log(tatm$x)) &
                (tatm$spr > 0.8+0.3*log(tatm$x)))
spike=subset(tatm, x %in% tatm$x[spike_ind])
# spike=subset(tatm, x %in% tatm$x[which((tatm$spl > 1.8) &
#                                       (tatm$spr > 2.4))])
points(spike$x, spike$freq, pch="O", col="grey", cex=1.2)
nospike=subset(tatm, !(x %in% tatm$x[spike_ind]))
nospike=subset(nospike, x < 2000)

# plot(nospike$x, nospike$freq, type="b", log="xy",
#         xlim=c(1, 1000))

# ta_mod=glm(log(freq) ~ x, data=nospike, subset=1:400)
# pred=predict(ta_mod, newdata=data.frame(x=1:500))
# lines(exp(pred))

ta_mod=gnm(freq ~ instances(Mult(1, Exp(x)), 2)-1,
                data=nospike, verbose=FALSE,
                start=c(2000.0, -0.04, 200.0, -0.001),
                family=poisson(link="identity"))
# print(summary(ta_mod))

exp_coef=as.numeric(coef(ta_mod))

lines(nospike$x, exp_coef[1]*exp(exp_coef[2]*nospike$x), col=pal_col[4])
lines(nospike$x, exp_coef[3]*exp(exp_coef[4]*nospike$x), col=pal_col[2])
pred=predict(ta_mod)
lines(nospike$x, pred, col=pal_col[1])

# c(exp_coef[2]/pred[1], exp_coef[4]/pred[1])

}

pal_col=rainbow(4)

fit_task_act(p615)

task_elapsed_time=function(df)
{
s_e=subset(df, start_time < end_time)
s_e$mins=(s_e$end_time-s_e$start_time)

c_mins=count(s_e$mins)
plot(c_mins, log="xy", col=pal_col[2],
        xlim=c(1, 600),
        xlab="Elapsed time (minutes)", ylab="Tasks\n")

x1=2:20
x2=21:240
dur_mod=glm(log(freq) ~ log(x), data=c_mins, subset=x1)
# print(summary(dur_mod))
pred=predict(dur_mod)
lines(x1, exp(pred), col=pal_col[1])

dur_mod=glm(log(freq) ~ log(x), data=c_mins, subset=x2)
# print(summary(dur_mod))
pred=predict(dur_mod, newdata=data.frame(x=x2))
lines(x2, exp(pred), col=pal_col[1])
}

task_start_time=function(start_time, col_str)
{
st15=count(start_time/60)
lines(st15$x, st15$freq/max(st15$freq, na.rm=TRUE), col=col_str)
}

same_day=subset(tf, start_date == end_date)

# nrow(same_day)/nrow(tf)

# space separates date from time
se_tf=subset(same_day, grepl(" ", task_actual_start_date) &
                        grepl(" ", task_actual_complete_date))

se_tf$start_time=get_day_time(se_tf$task_actual_start_date)
se_tf$end_time=get_day_time(se_tf$task_actual_complete_date)

se_tf$start_15time=15*round(se_tf$start_time/15)

s_e=subset(se_tf, start_time < end_time)
s_e$mins=(s_e$end_time-s_e$start_time)

pal_col=rainbow(2)

task_elapsed_time(subset(se_tf, project_key == 615))

@
\begin{figure}
\begin{center}
<<label=task_freq,fig=TRUE,echo=FALSE>>=
<<task-actual_freq>>
@
\end{center}
\caption{Both plots based on data from project 615.  Left: Number of tasks worked on for a given amount of actual time, red line is a fitted bi-exponential (components in purple and green), grey circles specify spike values (which were excluded from the fitted model);  right: Number of tasks completed on the same day taking a given elapsed time, with two fitted power-laws (exponent up to 20 minutes is -0.3, and -1.3 after).}
\label{task_freq:fig}
\end{figure}


<<label=actual-4-est,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

# Spread of actual values for a given estimated value.

act_for_planned=function(plan_time, col_str=point_col)
{
h1=subset(tf, task_plan_time_minutes == plan_time)

ch1=count(h1$task_actual_time_minutes)

lines(ch1, col=col_str)
}

pal_col=rainbow(3)

plot(0.5, log="y",
        xlim=c(1, 180), ylim=c(1, 500),
        xlab="Actual time (minutes)", ylab="Tasks\n")

act_for_planned(30, pal_col[1])
act_for_planned(60, pal_col[2])
act_for_planned(120, pal_col[3])

legend(x="topright", legend=paste0("Estimated = ", c(30, 60, 120)), bty="n", fill=pal_col, cex=1.3)

# Spread of estimated values for a given actual value.

planned_for_act=function(actual_time, col_str=point_col)
{
h1=subset(tf, task_actual_time_minutes == actual_time)

ch1=count(round(h1$task_plan_time_minutes))

lines(ch1, col=col_str)
}

pal_col=rainbow(3)

plot(0.5, log="y",
        xlim=c(1, 180), ylim=c(1, 500),
        xlab="Estimated time (minutes)", ylab="Tasks\n")

planned_for_act(30, pal_col[1])
planned_for_act(60, pal_col[2])
planned_for_act(120, pal_col[3])

legend(x="topright", legend=paste0("Actual = ", c(30, 60, 120)), bty="n", fill=pal_col, cex=1.3)

@
\begin{figure}
\begin{center}
<<label=act_4_est,fig=TRUE,echo=FALSE>>=
<<actual-4-est>>
@
\end{center}
\caption{Both plots are based on data for all projects.  Left: Number of tasks requiring a given actual time for the same amount of estimated time; right: Number of tasks estimated to take a given amount of time, having a given actual time to implement.}
\label{act_4_est:fig}
\end{figure}


<<label=task-interrupt_freq,include=FALSE,echo=FALSE>>=
plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(3)

interrupt_c=count(timf$time_log_interrupt_minutes)
plot(interrupt_c, log="xy", col=pal_col[2],
        xlim=c(1, 500), ylim=c(1, 1e4),
        xlab="Interval (minutes)", ylab="Tasks\n")

text(10, 950, "10", col=pal_col[3], cex=1.2)
text(15, 600, "15", col=pal_col[3], cex=1.2)
text(20, 400, "20", col=pal_col[3], cex=1.2)
text(30, 250, "30", col=pal_col[3], cex=1.2)
text(60, 200, "60", col=pal_col[3], cex=1.2)
text(90, 60, "90", col=pal_col[3], cex=1.2)
text(120, 80, "120", col=pal_col[3], cex=1.2)
text(180, 45, "180", col=pal_col[3], cex=1.2)

x_vals=5:100
int5_mod=glm(log(freq) ~ log(x), data=interrupt_c, subset=x_vals)
# summary(int5_mod)

pred=predict(int5_mod, newdata=data.frame(x=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])


# Percentage interrupted sessions for each project
interrupt_perc=function(df)
{
return(100-length(which(df$time_log_interrupt_minutes == 0))/nrow(df)*100)
}

proj_int=ddply(timf, .(project_key), interrupt_perc)

plot(sort(proj_int$V1), col=point_col,
        xaxs="i",
        xlab="Project", ylab="Work sessions interrupted (sorted percent)\n")

@
\begin{figure}
\begin{center}
<<label=interrupt_freq,fig=TRUE,echo=FALSE>>=
<<task-interrupt_freq>>
@
\end{center}
\caption{Left: Number of task work sessions whose interruptions consumed a given amount of actual time; red line is a fitted power law of the form: $\mathit{freq} \propto \mathit{interval}^{-1.8}$, for all work sessions excluding spikes at round-numbers (some highlighted in blue); right: percentage of work sessions experiencing at least one recorded interruption, sorted by percentage.}
\label{interrupt_freq:fig}
\end{figure}


<<label=task-per_person,include=FALSE,echo=FALSE>>=
plot_layout(1, 2)

par(mar=MAR_default+c(0.0, 1.2, 0, 0)) # leave more room for label

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(5)

# Number of tasks implemented by individuals working on a given project
proj_per_tasks=function(df, col_str)
{
person_info=ddply(df, .(person_key), person_totals)
lines(sort(person_info$num_tasks), col=col_str)
}

plot(1, type="n", log="y",
	xaxs="i", yaxs="i",
	xlim=c(0, 47), ylim=c(1, 1200),
	xlab="Person", ylab="Tasks (sorted)\n")

dum=proj_per_tasks(p615, pal_col[1])
dum=proj_per_tasks(p614, pal_col[2])
dum=proj_per_tasks(p617, pal_col[3])
dum=proj_per_tasks(p95, pal_col[4])

legend(x="bottomright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)

# What is the relationship between number of tasks a person implements
# and the total time they spend implementing them?
pal_col=rainbow(2)

person_info=ddply(tf, .(person_key), person_totals)
x_range=1:max(person_info$num_tasks)

or=order(person_info$num_tasks)
plot(person_info$num_tasks[or], person_info$act_total[or], log="xy", col=pal_col[2],
        xaxs="i",
        xlim=range(x_range),
        xlab="Tasks", ylab="Implementation time (mins)\n\n")

per_mod=glm(log(act_total) ~ log(num_tasks), data=person_info)
# summary(per_mod)

pred=predict(per_mod, newdata=data.frame(num_tasks=x_range))
lines(x_range, exp(pred), col=pal_col[1])

@
\begin{figure}
\begin{center}
<<label=task_person,fig=TRUE,echo=FALSE>>=
<<task-per_person>>
@
\end{center}
\caption{Left: Number of tasks (in sorted order) implemented on projects 614, 615, 617 and 95, by each individual working on it; right: For all CESAW projects, the number of tasks each individual implemented against the total implementation time for those tasks, red line is a fitted power law with exponent 0.88.}
\label{task_person:fig}
\end{figure}


<<label=task_elapsed-time,include=FALSE,echo=FALSE>>=
# Elapsed time for task completed within same day

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

same_day=subset(tf, start_date == end_date)

# nrow(same_day)/nrow(tf)

# space separates date from time
se_tf=subset(same_day, grepl(" ", task_actual_start_date) &
			grepl(" ", task_actual_complete_date))

se_tf$start_time=get_day_time(se_tf$task_actual_start_date)
se_tf$end_time=get_day_time(se_tf$task_actual_complete_date)

se_tf$start_15time=15*round(se_tf$start_time/15)

s_e=subset(se_tf, start_time < end_time)
s_e$mins=(s_e$end_time-s_e$start_time)

# Normalised number of tasks started at a given time of day

pal_col=rainbow(4)
plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(7, 23), ylim=c(0, 1),
	xlab="Time of day (centered)", ylab="Tasks (normalised)\n")

legend(x="topright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)

# Original time was mapped to MST, and need to map it back to center time of day
task_start_time(subset(se_tf, project_key == 615)$start_15time-1*60, pal_col[1])
task_start_time(subset(se_tf, project_key == 614)$start_15time-1*60, pal_col[2])
task_start_time(subset(se_tf, project_key == 617)$start_15time-1*60, pal_col[3])
task_start_time(subset(se_tf, project_key == 95)$start_15time+6*60, pal_col[4])

# Number of minutes between two sessions, for the same task, where
# the sessions have the same start day
session_break=function(df)
{
if (nrow(df) != 2)
   return(NULL)
if (df$start_date[1] != df$start_date[2])
   return(NULL)
ind=order(df$start_time)
return(df$start_time[ind[2]]-df$end_time[ind[1]])
}

sday_sess_break=ddply(se_tf, .(wbs_element_key), session_break)

# cs_break=count(sday_sess_break$V1)
# cs_break=subset(cs_break, x > 0)
# plot(cs_break, log="xy")

cs_15break=count(15*round(sday_sess_break$V1/15))
cs_15break=subset(cs_15break, x > 0)
plot(cs_15break, log="xy", col=point_col,
        xlab="Separation interval (minutes)", ylab="Session pairs\n")

@
\begin{figure}
\begin{center}
<<label=elapsed-time,fig=TRUE,echo=FALSE>>=
<<task_elapsed-time>>
@
\end{center}
\caption{Left: For tasks started and completed on the same day, number of tasks starting at a given centered time of day, within a 15-minute window for projects 614, 615, 617 and 95; right: For two sessions, of the same task, starting on the same day, the number of session intervals of a given length (in 15-minute bins).}
\label{elapsed-time:fig}
\end{figure}


<<label=large-delta_mins,include=FALSE,echo=FALSE>>=

plot_layout(1, 2)

par(cex.axis=1.3)
par(cex.lab=1.5)

pal_col=rainbow(2)

tdelta=count(timf$time_log_delta_minutes)
plot(tdelta, log="y", col=pal_col[2],
        xlim=c(100, 1300),
        xlab="Minutes", ylab="Time-log occurrences\n")

text(120, 2600, "120", col=pal_col[1], cex=1.2)
text(180, 1600, "180", col=pal_col[1], cex=1.2)
text(240, 540, "240", col=pal_col[1], cex=1.2)
text(360, 160, "360", col=pal_col[1], cex=1.2)
text(480, 65, "480", col=pal_col[1], cex=1.2)
text(600, 35, "600", col=pal_col[1], cex=1.2)
text(720, 25, "720", col=pal_col[1], cex=1.2)
text(900, 60, "900", col=pal_col[1], cex=1.2)
text(1200, 6.5, "1200", col=pal_col[1], cex=1.2)

# length(which(timf$time_log_delta_minutes > 600))/nrow(timf)
# [1] 0.000751396
# length(which(timf$time_log_delta_minutes > 420))/nrow(timf)
# [1] 0.001880945
# length(which(timf$time_log_delta_minutes > 360))/nrow(timf)
# [1] 0.002701097

pal_col=rainbow(2)

tdelta=count(timf$time_log_delta_minutes)
tfnz=subset(tf, task_plan_time_minutes > 0)
# Count ratios using equal sized bins, which for a log scale
# requires geometrically growing sizes.
ea_bins=cut(log(tfnz$task_actual_time_minutes/tfnz$task_plan_time_minutes), breaks=100)
testact=count(ea_bins)
# Average lower+upper bound of bins created by cut
testact$lab=as.character(testact$x)
testact$midpt=(as.numeric(sub("\\((.+),.*", "\\1", testact$lab))+
                as.numeric(sub("[^,]*,([^]]*)\\]", "\\1", testact$lab)))/2

plot(testact$midpt, log(testact$freq), col=pal_col[2],
                xaxt="n", yaxt="n",
                yaxs="i",
                ylim=log(c(1, 1e4)),
                xlab="Actual/Planned", ylab="Tasks\n")
x_pts=c(0.001, 0.01, 0.1, 1, 10, 100)
axis(1, at=log(x_pts), label=x_pts)
y_pts=c(1, 10, 100, 1000, 10000)
axis(2, at=log(y_pts), label=y_pts)

gt1=subset(testact, midpt > 0)
gt_mod=glm(log(freq) ~ midpt, data=gt1)
# summary(gt_mod)
pred=predict(gt_mod)
lines(gt1$midpt, pred, col=pal_col[1])

lt1=subset(testact, midpt < 0)
lt_mod=glm(log(freq) ~ midpt, data=lt1)
# summary(lt_mod)
pred=predict(lt_mod)
lines(lt1$midpt, pred, col=pal_col[1])

@
\begin{figure}
\begin{center}
<<label=delta_mins,fig=TRUE,echo=FALSE>>=
<<large-delta_mins>>
@
\end{center}
\caption{Both plots are based on data from all projects.  Left: Number of tasks taking a given number of delta minute in the time log; right: Number of tasks taking a given actual to estimated ratio (ratios aggregated into 100 bins), red lines are power law fits with exponents 1.3 and -1.6.}
\label{delta_mins:fig}
\end{figure}

