#
# fishing.R, 23 Apr 21

source("ESEUR_config.r")


library("bizdays")
library("gnm")
library("lubridate")
library("plyr")


# project_key     foreign key, unique project ID
# person_key      foreign key, unique developer ID
# team_key        foreign key, unique team ID
# wbs_element_key foreign key, unique work breakdown structure element ID
# plan_item_key   foreign key, unique  ID for the planned item associated with the task
# task_actual_start_date  time stamp for first work on task
# task_actual_complete_date       time stame when task was completed
# task_actual_time_minutes        total effort actually logged for the task
# task_plan_time_minutes  total effort planned for the task
# phase_key       unique key for the process phase in which the task work was done
# phase_short_name        readable name for process workflow phase
# process_name    readable name for process workflow

# Construction phases
c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

# Task facts
tf=read.csv("data/CESAW_task_fact.csv.xz", as.is=TRUE)
# bf=read.csv("data/basic_task_fact_sheet.csv", as.is=TRUE)

tf$start_date=as.Date(tf$task_actual_start_date, format="%m/%d/%Y")
tf$end_date=as.Date(tf$task_actual_complete_date, format="%m/%d/%Y")

tf$is_construct=(tf$phase_short_name %in% c_phase)

# length(which(tf$start_date > tf$end_date))
# [1] 638
# length(which(tf$start_date > tf$end_date))/nrow(tf)
# [1] 0.01032079

# table(tf$project_key)

plot(tf$task_actual_time_minutes, tf$task_plan_time_minutes, log="xy")

tf_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
						+factor(person_key)
						+factor(team_key)
						+phase_key,
			subset=(task_plan_time_minutes!=0), data=tf)
# summary(tf_mod)

# The actual times for particular values of planned time
planned_actual=function(p_time)
{
p30=subset(tf, task_plan_time_minutes == p_time)
cnt_p30=count(p30$task_actual_time_minutes)

plot(cnt_p30, type="b", log="xy")
}

# planned_actual(30)
# planned_actual(60)


# Model actual for a given Phase
model_phase=function(df, phase_str)
{
ci=subset(df, phase_short_name == phase_str)
points(ci$task_plan_time_minutes, ci$task_actual_time_minutes)

tf_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
						+factor(person_key),
                        subset=(task_plan_time_minutes!=0), data=ci)
print(summary(tf_mod))
}


# Projects with the most rows
p95=subset(tf, project_key == 95)   # 2nd
p614=subset(tf, project_key == 614) # 4th
p615=subset(tf, project_key == 615) # 1st
p617=subset(tf, project_key == 617) # 3rd

# Remove outliers
p614=subset(p614, start_date > as.Date("2014-06-18"))
p615=subset(p615, start_date > as.Date("2010-01-01"))

# plot(p615$task_plan_time_minutes, p615$task_actual_time_minutes, log="xy")
# 
# plot(type="n", p615$task_plan_time_minutes, p615$task_actual_time_minutes, log="xy")
# 
# model_phase(p615, "HLD Inspect")
# model_phase(p615, "Reqts Inspect")
# model_phase(p615, "Design Inspect")
# model_phase(p615, "Code Inspect")

# How many people work on a project?
proj_p_cnt=ddply(tf, .(project_key), function(df) length(unique(df$person_key)))
proj_min_people=13

pal_col=rainbow(length(which(proj_p_cnt$V1 > proj_min_people)))

# How many people work on a wbs?
project_wbs=function(df)
{
wbs_people=ddply(df, .(wbs_element_key),
			function(df) length(unique(df$person_key)))
total_people=count(wbs_people$V1)
if (length(unique(df$person_key)) > proj_min_people)
   {
   col_count <<- col_count+1
   lines(total_people, col=pal_col[col_count])
   }

return(total_people)
}

plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(1, 6), ylim=c(1, 500),
	xlab="People working on wbs", ylab="wbs\n")
col_count=0

proj_wbs=ddply(tf, .(project_key), project_wbs)


# Summary information about a wbs
wbs_totals=function(df)
{
return(data.frame(
			num_tasks=nrow(df),
			num_persons=length(unique(df$person_key)),
			persons=paste0(sort(unique(df$person_key)), collapse=","),
			num_phases=length(unique(df$phase_short_name)),
			phases=paste0(sort(unique(df$phase_short_name)), collapse=","),
			# phases=paste0(unique(df$phase_short_name), collapse=","),
			est_total=sum(df$task_plan_time_minutes),
			act_total=sum(df$task_actual_time_minutes),
			total_days=sum(df$task_actual_time_minutes)/(60*7),
			start_date=min(df$start_date, na.rm=TRUE),
			end_date=max(df$end_date, na.rm=TRUE),
			stringsAsFactors=FALSE
			))
}


# Summary information about a person
person_totals=function(df)
{
return(data.frame(person_key=df$person_key[1],
			num_tasks=nrow(df),
			num_wbs=length(unique(df$wbs_element_key)),
			num_phases=length(unique(df$phase_short_name)),
			unq_est=length(unique(df$task_plan_time_minutes))/nrow(df),
			unq_act=length(unique(df$task_actual_time_minutes))/nrow(df),
			act_total=sum(df$task_actual_time_minutes),
			total_days=sum(df$task_actual_time_minutes)/(60*7),
			stringsAsFactors=FALSE
			))
}

# Summary information about a project
project_totals=function(df)
{
if (nrow(df) < 200)
   return(NULL)
person_info=ddply(df, .(person_key), person_totals)
return(data.frame(num_people=length(unique(df$person_key)),
			num_tasks=nrow(df),
			num_wbs=length(unique(df$wbs_element_key)),
			num_phases=length(unique(df$phase_short_name)),
			p_phase_avg=mean(person_info$num_phases),
			act_total=sum(df$task_actual_time_minutes),
			stringsAsFactors=FALSE
			))
}


project_info=ddply(tf, .(project_key), project_totals)

plot(sort(project_info$num_wbs), col=point_col,
	xlab="Project", ylab="wbs implemented\n")

person_info=ddply(p615, .(person_key), person_totals)
# table(person_info$num_phases)
# table(person_info$num_tasks)

# plot(person_info$num_tasks, person_info$act_total, log="xy")
# plot(person_info$num_phases, person_info$act_total, log="y")
# plot(person_info$num_tasks, person_info$num_phases)
# 
# plot(sort(person_info$total_days))

# What is the relationship between number of tasks a person implements
# and the total time they spend implementing them?

pal_col=rainbow(2)

person_info=ddply(tf, .(person_key), person_totals)
x_range=1:max(person_info$num_tasks)

or=order(person_info$num_tasks)
plot(person_info$num_tasks[or], person_info$act_total[or], log="xy", col=pal_col[2],
        xaxs="i",
        xlim=range(x_range),
        xlab="Tasks", ylab="Implementation time\n")

per_mod=glm(log(act_total) ~ log(num_tasks), data=person_info)
summary(per_mod)

pred=predict(per_mod, newdata=data.frame(num_tasks=x_range))
lines(x_range, exp(pred), col=pal_col[1])

# What is the diversity of a person's estimates and actuals?

p10=subset(person_info, num_tasks > 20)

plot(sort(p10$unq_act), col=pal_col[1],
	yaxs="i",
	ylim=c(0.2, 1),
	xlab="Person", ylab="Unique values (fraction)\n")
points(sort(p10$unq_est), col=pal_col[2])

legend(x="bottomright", legend=c("Actual", "Estimated"), bty="n", fill=pal_col, cex=1.2)

# Sorted by estimate fraction order
est_order=order(p10$unq_est)

plot(p10$unq_act[est_order], col=pal_col[1],
        yaxs="i",
        ylim=c(0.2, 1),
        xlab="Person", ylab="Unique values (fraction)\n")
points(p10$unq_est[est_order], col=pal_col[2])

legend(x="bottomright", legend=c("Actual", "Estimated"), bty="n", fill=pal_col, cex=1.3)

wbs_info=ddply(p617, .(wbs_element_key), wbs_totals)

# table(wbs_info$num_persons)
# table(wbs_info$num_tasks)
# table(wbs_info$num_phases)

# Ident,Ident Inspect,Inspect - Author,Inspect - Others,IT,Work 
# 117 
# table(subset(wbs_info, num_phases == 5)$phases)

# Those tasks contained in wbs involving less than 10 tasks
# wbs10=subset(p615, wbs_element_key %in% subset(wbs_info, num_tasks < 10)$wbs_element)

# Are non-Misc dirty data?
# wbs1=subset(p615, wbs_element_key %in% subset(wbs_info, num_tasks ==1)$wbs_element)
# wbs2=subset(p615, wbs_element_key %in% subset(wbs_info, num_tasks ==2)$wbs_element)


# time_log_fact_key       foreign key to time log entry
# organization    Organization identifier
# project_key     foreign key to project id
# person_key      foreign key to person id
# team_key        foreign key to work breakdown structure id
# wbs_element_key foreign key to team id
# plan_item_key   foreign key to plan item
# time_log_start_date     time stamp for begin work
# time_log_end_date       time stamp for end work
# time_log_delta_minutes  stop_time-start_time- interrupt time
# time_log_interrupt_minutes      time (in minutes, not working)
# phase_key       foreign key to phase id
# phase_short_name        descriptive name for activity
# phase.process_key       foreign key to phase.process id
# process_name    descriptive name for the workflow used

# Build task facts based on time facts
task_times=function(df)
{
task_inf=data.frame(num_sess=nrow(df),
			day_diff=0,
			person_key=df$person_key[1],
			plan_item_key=df$plan_item_key[1],
			wbs_element_key=df$wbs_element_key[1],
			phase_short_name=df$phase_short_name[1],
			is_construct=df$is_construct[1],
			start_date=min(df$start_date, na.rm=TRUE),
			end_date=max(df$end_date, na.rm=TRUE),
			total_minutes=sum(df$time_log_delta_minutes, na.rm=TRUE),
			interrupt_minutes=sum(df$time_log_interrupt_minutes, na.rm=TRUE),
			stringsAsFactors=FALSE
			)
task_inf$day_diff=as.integer(task_inf$end_date-task_inf$start_date)

return(task_inf)
}

# Time facts about each task work session
timf=read.csv("data/CESAW_time_fact.csv.xz", as.is=TRUE)

timf$start_date=as.Date(timf$time_log_start_date, format="%m/%d/%Y")
timf$end_date=as.Date(timf$time_log_end_date, format="%m/%d/%Y")

timf$is_construct=(timf$phase_short_name %in% c_phase)


# Merge Task and Time information
merge_plan_time=function(df)
{
tptm=data.frame(person_key=df$person_key,
		plan_item_key=df$plan_item_key,
		task_actual_time_minutes=df$task_actual_time_minutes,
		task_plan_time_minutes=df$task_plan_time_minutes)

t=merge(tasks, tptm)

return(t)
}


# act_mod=glm(log(total_minutes) ~ log(task_plan_time_minutes)

# Tasks contained in wbs involving less than 10 tasks
# twbs15=subset(t615, wbs_element_key %in% subset(wbs_info, num_tasks < 15)$wbs_element)
# (person_key, plan_item_key) provides a unique combination for a task
# tasks=ddply(twbs15, .(person_key, plan_item_key), task_times)

tproj=subset(timf, project_key == 95)
tasks=ddply(tproj, .(person_key, plan_item_key), task_times)
pt=merge_plan_time(p95)

tproj=subset(timf, project_key == 614)
tasks=ddply(tproj, .(person_key, plan_item_key), task_times)
pt=merge_plan_time(p614)

tproj=subset(timf, project_key == 615)
tasks=ddply(tproj, .(person_key, plan_item_key), task_times)
pt=merge_plan_time(p615)

tproj=subset(timf, project_key == 617)
tasks=ddply(tproj, .(person_key, plan_item_key), task_times)
pt=merge_plan_time(p617)

# table(tasks$num_sess)
# table(tasks$day_diff)
# which(pt$total_minutes != pt$task_actual_time_minutes)

plot(count(tasks$day_diff), log="y", col="red",
	xaxs="i", yaxs="i",
	xlim=c(0, 60), ylim=c(1, 10000))
points(count(tasks$num_sess), col="blue")

legend(x="topright", legend=c("Days diff", "Task splits"), bty="n", fill=c("red", "blue"), cex=1.2)

# dev.copy(device=pdf, file="/usr1/rbook/ddiff_tsplits.pdf")
# dev.off()

# plot(tasks$num_sess, tasks$day_diff, log="xy")
# plot(tasks$num_sess, tasks$total_minutes, log="xy")

# plot(pt$task_plan_time_minutes, pt$day_diff, log="xy")
# plot(pt$task_plan_time_minutes, pt$num_sess, log="xy")


# act_mod=glm(log(total_minutes) ~ log(task_plan_time_minutes)
# 					, data=pt,
# 					subset=(task_plan_time_minutes > 0))
# act_mod=glm(log(total_minutes) ~ log(task_plan_time_minutes)
# 					+log(num_sess), data=pt,
# 					subset=(task_plan_time_minutes > 0))
# act_mod=glm(log(total_minutes) ~ (log(task_plan_time_minutes)
# 					+log(num_sess))^2, data=pt,
# 					subset=(task_plan_time_minutes > 0))
# act_mod=glm(log(total_minutes) ~ log(task_plan_time_minutes)
# 					+log(num_sess)
# 					+log(day_diff+1e-3), data=pt,
# 					subset=(task_plan_time_minutes > 0))

# Fit estimation model for particular project
act_mod=glm(log(total_minutes) ~ log(task_plan_time_minutes)
					+(day_diff==0)
					+log(num_sess)
					# +is_construct
					+phase_short_name
					+(interrupt_minutes == 0)
					# +log(interrupt_minutes+1)
					, data=pt,
					subset=(task_plan_time_minutes > 0))

summary(act_mod)


# How many tasks does a person have inflight on any day?
tasks_per_day=function(df)
{
p_days=day_zero # Start with all project days zero

sum_days=function(df)
   {
# The end date may be later, assume stop button hit the next day
   day_range=1+as.integer(df$start_date-p_start_date)
   p_days[day_range]<<-p_days[day_range]+1
   return(NULL)
   }

a_ply(df, 1, sum_days)
return(p_days)
}

p_start_date=min(tproj$start_date) # Start of project
p_start_day=wday(p_start_date, week_start=1)
p_num_days=1+as.integer(max(tproj$end_date)-p_start_date)
day_zero=rep(0, p_num_days) # Zero every project day

pt_per_day=daply(tproj, .(person_key), tasks_per_day)

total_per_day=colSums(pt_per_day)

t=total_per_day
t[t == 0]=0.7 # so log does not complain
plot(t, log="y", col=point_col,
	xaxs="i", yaxs="i",
	xlim=c(500, 2230),
        xlab="Days since project start", ylab="Tasks worked on\n")

acf(total_per_day)


wd_ind=seq(1, length(total_per_day), by=7)

pt_per_wday=sapply(0:6, function(X) sum(total_per_day[X+wd_ind], na.rm=TRUE))

plot(pt_per_wday, col=point_col,
        xaxs="i", yaxs="i",
        xaxt="n",
        xlim=c(0.9, 7.1),
        xlab="", ylab="Total tasks\n")
axis(1, at=1:7, label=c("Monday", "", "Wednesday", "", "Friday", "", "Sunday"))


# Fit a zero-truncated negative binomial distribution

plot(count(as.vector(pt_per_day)), log="y",
	xlim=c(1, 25), ylim=c(1, 2100),
	xlab="Tasks worked on during day", ylab="Days\n")


library(gamlss)
library(gamlss.tr)

# We need to explicitly specify where the distribution is to be truncated
gen.trun(par=0, family=NBII)


day_list=as.vector(pt_per_day)
day_list=day_list[(day_list > 0) & (day_list < 21)]

# Fit a zero-truncated, type II, negative binomial distribution
g_NBIItr=gamlss(day_list ~ 1, family=NBIItr)

# print(summary(g_NBIItr))

NBII_mu=exp(coef(g_NBIItr, "mu"))
NBII_sigma=exp(coef(g_NBIItr, "sigma"))

# print(c(NBII_mu, NBII_sigma))

lines(dNBIItr(1:20, mu=NBII_mu, sigma=NBII_sigma)*length(day_list), col="red")

AIC=as.numeric(AIC(g_NBIItr)
log_likelihood=as.numeric(logLik(g_NBIItr))


# How many tasks does a person have continued from previous days?
tasks_continued=function(df)
{
p_days=day_zero # Start with all project days zero

sum_days=function(df)
   {
   day_range=1+as.integer(seq(df$start_date+1, df$end_date, by=1)-p_start_date)
   p_days[day_range]<<-p_days[day_range]+1
   return(NULL)
   }

a_ply(subset(df, start_date < end_date), 1, sum_days)
return(p_days)
}


# How many tasks are continued for N days?
continuation_days=function(df)
{
proj=subset(df, (start_date <= end_date) &
			!(is.na(start_date) | is.na(end_date)))

# Create a US calendar without any holidays!
create.calendar("US", weekdays=c("saturday", "sunday"))

# There is a bizdays in forecast!
task_dur=bizdays::bizdays(proj$start_date, proj$end_date, "US")
task_freq=count(task_dur)
plot(task_freq, log="xy", col=pal_col[2],
	xlim=c(1, 500), ylim=c(1, 6500),
	xlab="Continuation working days", ylab="Tasks\n")

points(0.9, task_freq$freq[task_freq$x == 0], pch="C", cex=1.2, col="red")
tf_mod=glm(log(freq) ~ log(x), data=task_freq, subset=((x > 0) & (x < 10)))
# print(summary(tf_mod))
pred=predict(tf_mod)
lines(exp(pred), col=pal_col[1])

x_vals=10:59
tf_mod=glm(log(freq) ~ log(x), data=task_freq, subset=((x > 9) & (x < 60)))
# print(summary(tf_mod))
pred=predict(tf_mod, newdata=data.frame(x=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])

proj$tdays=task_dur
return(proj)
}

proj=continuation_days(p615)
# proj=continuation_days(p614)
# proj=continuation_days(p617)
# proj=continuation_days(p95)

proj=continuation_days(tasks)
proj=subset(proj, tdays > 1)
plot(proj$num_sess, proj$tdays, log="xy")

# Model the numbe rof continuation days against number of work sessions
ls_fit=loess.smooth(log(proj$tdays), log(proj$num_sess), span=0.3)
lines(exp(ls_fit$x), exp(ls_fit$y), col="green")

ts_mod=glm(log(tdays) ~ log(num_sess), data=proj)
summary(ts_mod)


# Any connection between task duration and day of the week?
proj$tdays=task_dur
proj$wday=wday(proj$start_date, week_start=1)

plot(proj$wday, proj$tdays, log="y")

wd_mod=glm(log(tdays) ~ wday, data=proj, subset=(tdays > 0))
summary(wd_mod)

sapply(1:5, function(X) mean(subset(proj, wday == X)$tdays))
# [1] 10.226447  7.952310  8.158093  8.577563 11.086716
sapply(1:5, function(X) mean(subset(proj, (wday == X) &
					(tdays < 10))$tdays))
# [1] 1.377138 1.286170 1.332834 1.461718 1.367003

proj=subset(proj, (tdays > 0) & (task_plan_time_minutes > 0))

# Planned task time and total actual elapsed days
plot(proj$task_plan_time_minutes, proj$tdays, log="xy",
	xlim=c(10, 200), ylim=c(5, 50))
smoothScatter(proj$task_plan_time_minutes, proj$tdays, log="xy")
smoothScatter(proj$task_plan_time_minutes, proj$tdays, log="xy", xlim=c(20, 3e3), ylim=c(5, 500)) 

smoothScatter(proj$task_actual_time_minutes, proj$tdays, log="xy", xlim=c(20, 3e3), ylim=c(5, 500)) 

# Which phases have the most tasks sessions?

ph_2sess=count(subset(tasks, num_sess == 2)$phase_short_name)
ph_Nsess=count(subset(tasks, num_sess > 30)$phase_short_name)


# Analysis of Work Breakdown Entries
wbs_person=function(df)
{
   # Phases in which an individual performed a task
   person_info=function(df)
   {
   return(data.frame(task_phases=paste0(unique(df$phase_short_name), collapse=","),
			num_tasks=nrow(df),
			stringsAsFactors=FALSE
			))
   }

# Ordering by start of task time should ensure correct ordering of phases
task_ord=df[order(df$start_time), ]
# Try to produce consistent sequences across wbs by mapping
# person_key to small range of values.
p_unq=unique(task_ord$person_key)
task_ord$person_ord=mapvalues(task_ord$person_key, p_unq, 1:length(p_unq))

p_tasks=ddply(task_ord, .(person_ord), person_info)
p_tasks$person_key=p_unq
p_tasks$total_tasks=sum(p_tasks$num_tasks)
# Paste numbers so we can count how many of each sequence occur
p_tasks$t_seq=paste0(sort(p_tasks$num_tasks, decreasing=TRUE), collapse=",")

return(p_tasks)
}

p615$start_time=as.POSIXct(p615$task_actual_start_date, format="%m/%d/%Y %H:%M")

wbs_info=ddply(p615, .(wbs_element_key), wbs_totals)

# wbs involving three people is over 60% of tasks
tp3=subset(wbs_info, num_persons==3)

# t1=sort(table(tp3$persons), decreasing=TRUE)
# sum(t1[1:10])
# table(tp3$num_tasks)
# sort(table(tp3$phases))

wbs_3p=subset(p615, wbs_element_key %in% tp3$wbs_element_key)
wbs_per_task=ddply(wbs_3p, .(wbs_element_key), wbs_person)
wbs_per_task=ddply(p615, .(wbs_element_key), wbs_person)

sort(table(wbs_per_task$t_seq)/length(unique(wbs_per_task$person_ord)))

wbs_10=subset(wbs_per_task, t_seq == "3,2,2")
wbs_10$task_phases

# table(wbs_10$task_phases, wbs_10$num_tasks)

# wbs_per_task[(1:3)+0*3,]

person_1=seq(1, nrow(wbs_per_task), by=3)


# Build graph of people who worked together on a project's wbs implementation

library("grid")  # needed for gpar
library("seriation")

pal_col=rainbow(7)

# Count the number of time each person works with another
# person on a wbs.
wbs_person_links=function(df)
{
u_person=unique(df$s_person)
num_persons=length(u_person)
if (num_persons == 1)
   return(NULL)

num_tasks=nrow(df)
act_total=sum(df$task_actual_time_minutes)

# Generate all pairs of people; each column is a pair
pairs=combn(u_person, 2)

t=sapply(1:dim(pairs)[2], function (X)
				{
				from_to=cbind(pairs[1, X], pairs[2, X])
				person_mat[from_to] <<- person_mat[from_to]+1
				from_to=cbind(pairs[2, X], pairs[1, X])
				person_mat[from_to] <<- person_mat[from_to]+1
				})
# return(NULL)
}


# Seriation matrix based on number of times people worked together
project_people_rel=function(proj, p_num)
{
# Build matrix for each person
u_proj_people=unique(proj$person_key)
total_people=length(u_proj_people)
person_mat <<- matrix(nrow=total_people, ncol=total_people, data=0)

# Map to values starting at one
proj$s_person=mapvalues(proj$person_key, u_proj_people, 1:length(u_proj_people))

d_ply(proj, .(wbs_element_key), wbs_person_links)

person_dist=dist(1/(person_mat+1e-5))
pser=seriate(person_dist)

pimage(person_dist, pser, key=FALSE, cex=1.0, col=rev(pal_col),
	newpage=FALSE, # needed to display multiple images within plot
        gp=gpar(cex=0.8), # needs library(grid)
	main=p_num,
        xlab="Person", ylab="Person")

return(NULL)
}

par(mar=MAR_default-c(2.7, 3.9, 0.7, 0.7))

# Yuk.  Only way to get more than one image in a plot
grid.newpage()
pushViewport(viewport(layout=grid.layout(nrow=2, ncol=2)))

pushViewport(viewport(layout.pos.row=1, layout.pos.col=1))
project_people_rel(p614, "614")
popViewport()
pushViewport(viewport(layout.pos.row=1, layout.pos.col=2))
project_people_rel(p615, "615")
popViewport()
pushViewport(viewport(layout.pos.row=2, layout.pos.col=1))
project_people_rel(p617, "617")
popViewport()
pushViewport(viewport(layout.pos.row=2, layout.pos.col=2))
project_people_rel(p95, "95")
popViewport(2)


# Model factors that appear to affect how many people someone works with

person_totals=function(df, ww)
{
return(data.frame(person_key=df$person_key[1],
			num_other_p=length(which(ww[df$s_person[1], ] !=0)),
                        num_tasks=nrow(df),
                        num_wbs=length(unique(df$wbs_element_key)),
                        num_phases=length(unique(df$phase_short_name)),
                        act_total=sum(df$task_actual_time_minutes),
                        total_days=sum(df$task_actual_time_minutes)/(60*7),
                        stringsAsFactors=FALSE
                        ))
}

# Count people working with somebody on each of their wbs
project_people_rel=function(proj)
{
person_mat <<- matrix(nrow=total_people, ncol=total_people, data=0)

d_ply(proj, .(wbs_element_key), wbs_person_links)

return(person_mat)
}

u_proj_people=unique(p615$person_key)
total_people=length(u_proj_people)
p615$s_person=mapvalues(p615$person_key, u_proj_people, 1:length(u_proj_people))


ww_p615=project_people_rel(p615)
person_info=ddply(p615, .(person_key), person_totals, ww_p615)

# should be a zero-truncated distribution
# num_wbs just does not provide a good-enough contribution.
# np_mod=glm(num_other_p ~ sqrt(num_tasks)*num_phases, data=person_info,
np_mod=glm(num_other_p ~ num_tasks*num_phases, data=person_info,
					family=poisson)
summary(np_mod)
# Count can never go below 1, use a zero truncated poisson, i.e., pospoisson
# library("VGAM")
# 
# ztnp_mod=vglm(num_other_p ~ sqrt(num_tasks)*num_phases, data=person_info,
# 					family=pospoisson)
# Fitted model is the same
# summary(ztnp_mod)


# Count the number of time a person worked on a task in each phase
people_phase=function(proj)
{
   count_phases=function(df)
   {
   tp=count(df$s_phase)
   person_mat[df$s_person[1], tp$x] <<- person_mat[df$s_person[1], tp$x]+tp$freq
   }

person_mat <<- matrix(nrow=total_people, ncol=total_phase, data=0)

d_ply(proj, .(person_key), count_phases)

return(person_mat)
}

# Seriation of people/phases
u_proj_people=unique(p615$person_key)
total_people=length(u_proj_people)
p615$s_person=mapvalues(p615$person_key, u_proj_people, 1:length(u_proj_people))
u_proj_phase=unique(p615$phase_short_name)
total_phase=length(u_proj_phase)
p615$s_phase=mapvalues(p615$phase_short_name, u_proj_phase, 1:length(u_proj_phase))

pal_col=rainbow(3)

pp_mat=people_phase(p615)

pp_inv=1/(pp_mat+1e-5)
pp_dist=dist(pp_inv)

ppser=seriate(pp_inv, method="PCA")

pimage(pp_inv, ppser, key=FALSE, cex=1.2, col=rev(pal_col),
	# newpage=FALSE, # needed to display multiple images within plot
        gp=gpar(cex=1.1), # needs library(grid)
        xlab="Phase", ylab="Person")


# Patterns in estimate values
# p95 # p614 # p615 # p617

pal_col=rainbow(2)

total_est=count(p615$task_plan_time_minutes)
plot(total_est, log="y", col=pal_col[2],
        xaxs="i",
        xlim=c(1, 480), ylim=c(12, 2500),
        xlab="Estimate (minutes)", ylab="Tasks\n")

# subset(total_est, freq > 160)

text(12, 450, "12", col=pal_col[1])
text(30, 2600, "30", col=pal_col[1])
text(43, 320, "43", col=pal_col[1])
text(60, 720, "60", col=pal_col[1])
# text(90, 220, "12", col=pal_col[1])
text(97, 280, "97", col=pal_col[1])
text(122, 370, "122", col=pal_col[1])
text(180, 280, "180", col=pal_col[1])
# text(240,  90, "12", col=pal_col[1])
text(275, 290, "275", col=pal_col[1])
text(383, 200, "383", col=pal_col[1])


# Fit models to the inspection estimates

inspect_est=function(ins_str, proj_df)
{
ins_df=subset(proj_df, phase_short_name == ins_str)
ins_df=subset(ins_df, (task_actual_time_minutes > 0) &
			(task_plan_time_minutes >0))

if (nrow(ins_df) < 2)
   return(NULL)
# Require that a person make some minimum number of estimates
pk=count(ins_df$person_key)
many_pk=subset(pk, freq > 4)
ins_df=subset(ins_df, person_key %in% many_pk$x)

# print(c(ins_str, nrow(ins_df)))

ins_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
						+num_sess
						+p_ID,
						, data=ins_df)
print(summary(ins_mod))

return(ins_mod)
}


# Return the person_key coefficients
pe_coef=function(p_mod, num_uper)
{
# we don't want the first two, three with num_sess
c_vals=exp(coef(p_mod))[-(1:3)]
pid=as.numeric(sub("p_ID", "", names(c_vals))) # get the numeric person ID

# print(pid)
t_coef=rep(NA, num_uper)
t_coef[pid]=c_vals

return(t_coef)
}

plot_coef=function(proj_df, task_str, kind_phase)
{
# To simplify model fitting map person_key to a small integer string.
uper=unique(proj_df$person_key)
proj_df$p_ID=as.character(mapvalues(proj_df$person_key, uper, 1:length(uper)))

# The class returned by glm is a list
task_mod=alply(task_str, 1, inspect_est, proj_df)

task_mod=compact(task_mod) # Remove NULL entries from list

# Get the p_ID coefficients from each fitted model
per_coef=laply(task_mod, pe_coef, length(uper))

plot(0.1, type="n", log="y",
	xaxs="i", yaxs="i",
	xlim=c(1, length(uper)+0.5), ylim=c(0.15, 4),
	xlab="Person", ylab="Coefficient\n")
dummy=sapply(1:nrow(per_coef),
				function(X)
				{
				points(per_coef[X, ], col=pal_col[1])
				})

dummy=sapply(1:ncol(per_coef), function(X)
				{
				lines(c(X, X), range(per_coef[ , X], na.rm=TRUE), col=pal_col[2])
				})

lines(c(1, 50), c(1, 1), col="grey")
text(ncol(per_coef)/2, 3.6, kind_phase, cex=1.2)
}

# c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")

pal_col=rainbow(2)

# insp_proj=p615
insp_proj=pt # needed for num_sess

# List of Inspect tasks
t=unique(pt$phase_short_name)
inspect_str=t[grep("Inspect", t)]

plot_coef(pt, inspect_str, "Inspection")
plot_coef(pt, c_phase, "Creation")

# Manual analysis of a couple of Inspect phases
io=subset(p615, phase_short_name == "Inspect - Others")

x_vals=1:200

plot(io$task_plan_time_minutes, io$task_actual_time_minutes, log="xy",
	xlab="Estimate", ylab="Actual")
lines(x_vals, x_vals, col=pal_col[2])

io_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes), data=io)
summary(io_mod)
pred=predict(io_mod, newdata=data.frame(task_plan_time_minutes=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])

ii=subset(p615, phase_short_name == "Ident Inspect")

plot(ii$task_plan_time_minutes, ii$task_actual_time_minutes, log="xy",
	xlab="Estimate", ylab="Actual")
lines(x_vals, x_vals, col=pal_col[2])

ii_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
						# +as.factor(person_key)
						, data=ii)
summary(ii_mod)
pred=predict(ii_mod, newdata=data.frame(task_plan_time_minutes=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])


# Look for patterns or structure in the wbs_element tree

library("igraph") 

pal_col=rainbow(3)

wtree=read.csv("data/wbs_parent.csv", as.is=TRUE)

# Pick a project
wp615=subset(wtree, project_key == 615)

uwp615=unique(wp615)
uw_from=unique(uwp615$parent_wbs_element_key)
uw_to=unique(uwp615$wbs_element_key)
uwp615$color=pal_col[2] # the default color, e.g., green
# leaf nodes don't have a to-node in the list of from-nodes 
uwp615$color[!(uwp615$wbs_element_key %in% uw_from)]=pal_col[3]

# graph.data.frame expects "from","to" in the first two columns
ant_g=graph.data.frame(data.frame(uwp615$parent_wbs_element_key, uwp615$wbs_element_key, color=uwp615$color),
			directed=TRUE)
V(ant_g)$label=NA
V(ant_g)$size=0.3
V(ant_g)$color=pal_col[1]
E(ant_g)$arrow.size=0.2
# E(ant_g)$color=pal_col[2]

plot(ant_g, layout=layout_as_tree,
        vertex.frame.color=pal_col[1])

# Distance, in vertices, between root and leaves
tdist=distances(ant_g, v=which(degree(ant_g, mode="in") == 0),
			to=which(degree(ant_g, mode="out") == 0))


# Add start date to parent of leaf's

pal_col=rainbow(7)

p615$syear=as.numeric(substr(p615$start_date, 3, 4))

uwp615$color=pal_col[2] # Non-leaf color
v_color=rep(pal_col[1], nrow(uwp615))
# leaf nodes don't have a to-node in the list of from-nodes 
# uwp615$color[!(uwp615$wbs_element_key %in% uw_from)]=pal_col[3]

# Pick one of each wbs_element_key, to get its start_year
u_p615=subset(p615, !duplicated(wbs_element_key))

uwp615$leaf_p=(uwp615$wbs_element_key %in% u_p615$wbs_element_key)
uwp615$color[uwp615$leaf_p]=pal_col[mapvalues(uwp615$wbs_element_key[uwp615$leaf_p], u_p615$wbs_element_key, u_p615$syear)-9+2]
v_color[uwp615$leaf_p]="black"

ant_g=graph.data.frame(data.frame(uwp615$parent_wbs_element_key, uwp615$wbs_element_key, color=uwp615$color),
                        directed=TRUE,
			vertices=data.frame(c(uwp615$wbs_element_key, "NA"), color=c(v_color, 0)))

V(ant_g)$label=NA
V(ant_g)$size=0.3
# V(ant_g)$color=pal_col[2]
E(ant_g)$arrow.size=0.2
# E(ant_g)$color=pal_col[2]

plot(ant_g, layout=layout_as_tree,
        vertex.frame.color=V(ant_g)$color)

legend(x="topleft", legend=c("Leaf", "Non-leaf", paste0("20", 10:14)), bty="n", fill=c("black", pal_col), cex=1.2)


# Timeline of each persons task involvement
person_timeline=function(p_df, p_num)
{
task_points=function(df)
   {
   points(df$start_date, df$person_ord, col=pal_col[df$person_ord[1]])
   }


p_unq=unique(p_df$person_key[order(p_df$start_date)])
pal_col=rainbow(length(p_unq))
p_df$person_ord=mapvalues(p_df$person_key, p_unq, 1:length(p_unq))

plot(p_df$start_date[1], 0, type="n",
        xaxs="i", yaxs="i",
        xlim=range(p_df$start_date, na.rm=TRUE), ylim=c(0.8, 0.2+length(p_unq)),
        xlab="Date", ylab="Person\n")

text(p_df$start_date[1]+5, max(p_df$person_ord), p_num, pos=1)

p_tasks=d_ply(p_df, .(person_ord), task_points)
}

person_timeline(p614, 614)
person_timeline(p615, 615)
person_timeline(p617, 617)
person_timeline(p95, 95)


# Time spent, per week, by a person on a project

person_weektime=function(p_df, col_str)
{
weektime=function(df)
   {
   wtime=ddply(df, .(person_ord), function(df) sum(df$task_actual_time_minutes, na.rm=TRUE))
   return(wtime)
   }

week_percent=function(df)
   {
   total_weeks=max(df$week_num, na.rm=TRUE)-min(df$week_num, na.rm=TRUE)+1
   total_mins=sum(df$V1, na.rm=TRUE)
   max_mins_week=max(df$V1, na.rm=TRUE)
   return(data.frame(total_weeks, total_mins, max_mins_week)) 
   }

p_unq=unique(p_df$person_key[order(p_df$start_date)])
pal_col=rainbow(length(p_unq))
p_df$person_ord=mapvalues(p_df$person_key, p_unq, 1:length(p_unq))

start_year=year(min(p_df$start_date, na.rm=TRUE))
p_df$week_num=(year(p_df$start_date)-start_year)*52+week(p_df$start_date)

p_wtime=ddply(p_df, .(week_num), weektime)

# plot(1, type="n", log="y",
# 	xlim=range(p_wtime$week_num, na.rm=TRUE), ylim=range(p_wtime$V1, na.rm=TRUE),
# 	xlab="Week", ylab="Total minutes")
pweek=ddply(p_wtime, .(person_ord), week_percent)
pweek$mins_per_week=pweek$total_mins/pweek$total_weeks
pweek=pweek[order(pweek$mins_per_week), ]
lines(pweek$mins_per_week/60, col=col_str)
# lines(pweek$max_mins_week/60, lty=2, col=col_str)

}

pal_col=rainbow(4)

plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(1, 50), ylim=c(1, 20),
	xlab="Person", ylab="Hours per week\n")

person_weektime(p614, pal_col[1])
person_weektime(p615, pal_col[2])
person_weektime(p617, pal_col[3])
person_weektime(p95, pal_col[4])

legend(x="bottomright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)


# Number of distinct people starting a task in each week

person_perweek=function(p_df, col_str)
{
start_year=year(min(p_df$start_date, na.rm=TRUE))
p_df$week_num=(year(p_df$start_date)-start_year)*52+week(p_df$start_date)

p_perweek=ddply(p_df, .(week_num), function(df) length(unique(df$person_key)))

lines(p_perweek$V1, col=col_str)
}

pal_col=rainbow(4)

plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(1, 230), ylim=c(0, 31),
	xlab="Week", ylab="People\n")

person_perweek(p614, pal_col[1])
person_perweek(p615, pal_col[2])
person_perweek(p617, pal_col[3])
person_perweek(p95, pal_col[4])

legend(x="topright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)


# Some data quality checks

# Time spent on a task
tdelta=count(timf$time_log_delta_minutes)
plot(tdelta, log="y", col=pal_col[2],
        xlim=c(100, 2.4e3),
        xlab="Minutes", ylab="Time log occurrences\n")

text(360, 140, "360", col=pal_col[1])
text(480, 65, "480", col=pal_col[1])
text(600, 35, "600", col=pal_col[1])
text(720, 25, "720", col=pal_col[1])
text(900, 60, "900", col=pal_col[1])
text(1200, 6.5, "1200", col=pal_col[1])

# length(which(timf$time_log_delta_minutes > 600))/nrow(timf)
# [1] 0.000751396
# length(which(timf$time_log_delta_minutes > 420))/nrow(timf)
# [1] 0.001880945
# length(which(timf$time_log_delta_minutes > 360))/nrow(timf)
# [1] 0.002701097


# Estimated time spent

tplanned=count(tf$task_plan_time_minutes)
plot(tplanned, log="xy", col=pal_col[2],
        xlim=c(120, 5e3),
        xlab="Planned minutes", ylab="Tasks\n")


# Actual/Estimated ratio

pal_col=rainbow(2)

tfnz=subset(tf, (task_plan_time_minutes > 0) &
                (task_actual_time_minutes < 360))
# Count ratios using equal sized bins, which for a log scale
# requires geometrically growing sizes.
ea_bins=cut(log(tfnz$task_actual_time_minutes/tfnz$task_plan_time_minutes), breaks=100)
testact=count(ea_bins)
# Average lower+upper bound of bins created by cut
testact$lab=as.character(testact$x)
testact$midpt=(as.numeric(sub("\\((.+),.*", "\\1", testact$lab))+
		as.numeric(sub("[^,]*,([^]]*)\\]", "\\1", testact$lab)))/2

plot(testact$midpt, log(testact$freq), col=pal_col[2],
		xaxt="n", yaxt="n",
		yaxs="i",
		ylim=log(c(1, 1e4)),
		xlab="Actual/Planned", ylab="Tasks\n")
x_pts=c(0.001, 0.01, 0.1, 1, 10, 100)
axis(1, at=log(x_pts), label=x_pts)
y_pts=c(1, 10, 100, 1000, 10000)
axis(2, at=log(y_pts), label=y_pts)

gt1=subset(testact, midpt > 0)
# midpt has already been log'ed
gt_mod=glm(log(freq) ~ midpt, data=gt1)
summary(gt_mod)
pred=predict(gt_mod)
lines(gt1$midpt, pred, col=pal_col[1])

lt1=subset(testact, midpt < 0)
lt_mod=glm(log(freq) ~ midpt, data=lt1)
summary(lt_mod)
pred=predict(lt_mod)
lines(lt1$midpt, pred, col=pal_col[1])


# Fit biexponential to task actual time

fit_task_act=function(df)
{
# tatm=count(df$task_plan_time_minutes)
tatm=count(df$task_actual_time_minutes)
plot(tatm$x, tatm$freq, log="y", col=pal_col[3],
	xlim=c(1, 1000),
	xlab="Actual time (minutes)", ylab="Tasks\n")

# Find 'spike' values
tatm$spl=c(head(tatm$freq, -1)/pmax(tail(tatm$freq, -1), 3), NA)
tatm$spr=c(NA, tail(tatm$freq, -1)/pmax(head(tatm$freq, -1), 3))
# tatm$dspl=c(NA, diff(tatm$spl))
# tatm$tsp=0.8+0.5*log(tatm$x)
# tatm$x[which(tatm$sp > 1+0.5*log(tatm$x))]

spike_ind=which((tatm$spl > 0.5+0.3*log(tatm$x)) &
		(tatm$spr > 0.8+0.3*log(tatm$x)))
spike=subset(tatm, x %in% tatm$x[spike_ind])
# spike=subset(tatm, x %in% tatm$x[which((tatm$spl > 1.8) &
# 					(tatm$spr > 2.4))])
points(spike$x, spike$freq, pch="*", col="yellow")
nospike=subset(tatm, !(x %in% tatm$x[spike_ind]))
nospike=subset(nospike, x < 2000)

# plot(nospike$x, nospike$freq, type="b", log="xy",
#         xlim=c(1, 1000))

# ta_mod=glm(log(freq) ~ x, data=nospike, subset=1:400)
# pred=predict(ta_mod, newdata=data.frame(x=1:500))
# lines(exp(pred))

ta_mod=gnm(freq ~ instances(Mult(1, Exp(x)), 2)-1,
                data=nospike, verbose=TRUE,
                start=c(2000.0, -0.04, 200.0, -0.001),
                family=poisson(link="identity"))
print(summary(ta_mod))

exp_coef=as.numeric(coef(ta_mod))

lines(nospike$x, exp_coef[1]*exp(exp_coef[2]*nospike$x), col=pal_col[4])
lines(nospike$x, exp_coef[3]*exp(exp_coef[4]*nospike$x), col=pal_col[2])
pred=predict(ta_mod)
lines(nospike$x, pred, col=pal_col[1])

# c(exp_coef[2]/pred[1], exp_coef[4]/pred[1])

}

unloadNamespace("igraph") # defines a gnm function
pal_col=rainbow(4)

fit_task_act(p615)
fit_task_act(p614)
fit_task_act(p617)
fit_task_act(p95)


# Model creation phases
nntf=subset(tf, (task_plan_time_minutes > 0) & (task_plan_time_minutes < 2100) &
		!is.na(task_plan_time_minutes))

# c_phase=c("Ident", "Reqts", "HLD", "Design", "Code", "Test Devel", "Documentation")
cp_tasks=subset(nntf, phase_short_name %in% c_phase)
fit_task_act(cp_tasks)

# and non-creation phases
ncp_tasks=subset(nntf, !(phase_short_name %in% c_phase))
fit_task_act(ncp_tasks)


# Plan time
tptm=count(cp_tasks$task_plan_time_minutes)
plot(tptm$x, tptm$freq, type="b", log="xy",
        xlim=c(1, 1000),
        xlab="Planned time (minutes)", ylab="Tasks\n")

# Bin data to reduce variability
ea_bins=cut(log(ncp_tasks$task_plan_time_minutes), breaks=100)
testact=count(ea_bins)
# Average lower+upper bound of bins created by cut
testact$lab=as.character(testact$x)
testact$midpt=(as.numeric(sub("\\((.+),.*", "\\1", testact$lab))+
                as.numeric(sub("[^,]*,([^]]*)\\]", "\\1", testact$lab)))/2

plot(testact$midpt, testact$freq, log="y", col=pal_col[2],
                xaxt="n", yaxt="n",
                yaxs="i",
                ylim=c(1, 2e3),
                xlab="Planned", ylab="Tasks\n")
x_pts=c(0.1, 1, 10, 100, 1000)
axis(1, at=log(x_pts), label=x_pts)
y_pts=c(1, 10, 100, 1000, 5000)
axis(2, at=y_pts, label=y_pts)


# Time at which phases are started
phase_start=function(df, y_off)
{
t=df$start_date[grepl(major_phase[y_off], df$phase_short_name)]
points(t, rep(1+0.07*y_off, length(t)), pch="|", col=pal_col[y_off])
}

major_phase=c("Req", "HLD", "Design", "Code", "Test")

pal_col=rainbow(length(major_phase))

plot(as.Date("2004-06-18"), 0, type="n",
	xlim=c(as.Date("2015-02-01"), as.Date("2015-06-01")), ylim=c(1, 3),
	xlab="Date", ylab="Phase")

dum=sapply(1:length(major_phase), function(X) phase_start(p614, X))


# Investigate multiple sessions on the same day
session_time=function(df)
{
if (nrow(df) != 2)
   return(NULL)
if (df$start_date[1] != df$start_date[2])
   return(NULL)
s_ord=order(df$start_time) # Session order
task_inf=data.frame(sess1=df$time_log_delta_minutes[s_ord[1]],
			sess2=df$time_log_delta_minutes[s_ord[2]],
                        sess_diff=df$start_time[s_ord[2]]-
					df$end_time[s_ord[1]],
                        person_key=df$person_key[1],
                        plan_item_key=df$plan_item_key[1],
                        wbs_element_key=df$wbs_element_key[1],
                        phase_short_name=df$phase_short_name[1],
                        is_construct=df$is_construct[1],
                        start_date=min(df$start_date),
                        stringsAsFactors=FALSE
                        )

return(task_inf)
}


timf$start_time=as.POSIXct(timf$time_log_start_date, format="%m/%d/%Y %H:%M")
timf$end_time=as.POSIXct(timf$time_log_end_date, format="%m/%d/%Y %H:%M")

tproj=subset(timf, project_key == 615)
day_sess=ddply(tproj, .(person_key, plan_item_key), session_time)

t=table(round(day_sess$sess_diff, digits=1))
# plot(t)
plot(t, xlim=c(0, 13))

dsp=subset(day_sess, (sess1 > 0) & (sess2 > 0))

plot(dsp$sess1, dsp$sess2, log="xy")
smoothScatter(dsp$sess1, dsp$sess2, log="xy")

# Quadratic is a slightly better fit
# dsp_mod=glm(log(sess2) ~ log(sess1)+I(log(sess1)^2), data=dsp)
dsp_mod=glm(log(sess2) ~ log(sess1), data=dsp)
summary(dsp_mod)


# Look for time of day patterns

pal_col=rainbow(2)

# Convert HH:MM to minutes
get_day_time=function(task_date)
{
s_datetime=unlist(strsplit(task_date, " "))
s_HM=strsplit(s_datetime[seq(2, length(s_datetime), by=2)], ":")
s_mins=sapply(s_HM, function(X) return(as.numeric(X[1])*60+as.numeric(X[2])))
return(s_mins)
}

same_day=subset(timf, start_date == end_date)

# space separates date from time
se_tf=subset(same_day, grepl(" ", time_log_start_date) &
			grepl(" ", time_log_end_date))

se_tf$start_time=get_day_time(se_tf$time_log_start_date)
se_tf$end_time=get_day_time(se_tf$time_log_end_date)

se_tf$start_15time=15*round(se_tf$start_time/15)

st15=count(se_tf$start_15time)
plot(st15)

st=count(se_tf$start_time)
plot(st)

et=count(se_tf$end_time)
points(et, col="red")

s_e=subset(se_tf, start_time < end_time)

plot(s_e$start_time/60, s_e$end_time/60,
	xaxs="i", yaxs="i",
	xlab="Start time", ylab="End time")

s_e$mins=(s_e$end_time-s_e$start_time)
plot(s_e$start_time/60, s_e$mins/60,
	xaxs="i", yaxs="i",
	xlab="Start time", ylab="Duration (hours)")

c_mins=count(s_e$mins)
plot(c_mins, log="xy", col=pal_col[2],
	xlim=c(1, 600),
	xlab="Duration (minutes)")


# Interrupt time, during each session


pal_col=rainbow(2)

# For time csv: end_time-start_time ==
#			time_log_delta_minutes+time_log_interrupt_minutes
length(which(timf$mins != timf$time_log_delta_minutes+timf$time_log_interrupt_minutes))

plot(s_e$time_log_delta_minutes, s_e$time_log_interrupt_minutes, log="xy")
int_mod=glm(log(time_log_interrupt_minutes) ~ log(time_log_delta_minutes),
			data=s_e, subset=((time_log_interrupt_minutes > 1) &
					(time_log_delta_minutes > 1)))
summary(int_mod)

c_act_mins=count(s_e$time_log_delta_minutes)
c_total_mins=count(s_e$time_log_delta_minutes+s_e$time_log_interrupt_minutes)
plot(c_total_mins, log="xy", col=pal_col[2],
        xlim=c(1, 600),
        xlab="Duration (minutes)")
points(c_act_mins, col=pal_col[2])

interrupt_c=count(timf$time_log_interrupt_minutes)
plot(interrupt_c, log="xy", col=pal_col[2],
	xlim=c(1, 500), ylim=c(1, 1e4),
	xlab="Interval (minutes)", ylab="Interruptions\n")

text(10, 950, "10", col=pal_col[1])
text(15, 600, "15", col=pal_col[1])
text(20, 400, "20", col=pal_col[1])
text(30, 250, "30", col=pal_col[1])
text(60, 200, "60", col=pal_col[1])
text(90, 60, "90", col=pal_col[1])
text(120, 80, "120", col=pal_col[1])
text(180, 45, "180", col=pal_col[1])

x_vals=5:100
int5_mod=glm(log(freq) ~ log(x), data=interrupt_c, subset=x_vals)
summary(int5_mod)

pred=predict(int5_mod, newdata=data.frame(x=x_vals))
lines(x_vals, exp(pred), col=pal_col[1])

plot(same_day$time_log_delta_minutes, same_day$time_log_interrupt_minutes, log="xy")


# Task interrupt time, as-in interval between sessions

# space separates date from time
se_tf=subset(timf, grepl(" ", time_log_start_date) &
			grepl(" ", time_log_end_date))

se_tf$start_time=get_day_time(se_tf$time_log_start_date)
se_tf$end_time=get_day_time(se_tf$time_log_end_date)

# Number of minutes between two sessions, for the same task, where
# the sessions have the same start day
session_break=function(df)
{
if (nrow(df) != 2)
   return(NULL)
if (df$start_date[1] != df$start_date[2])
   return(NULL)
ind=order(df$start_time)
return(df$start_time[ind[2]]-df$end_time[ind[1]])
}

sday_sess_break=ddply(se_tf, .(wbs_element_key), session_break)

# cs_break=count(sday_sess_break$V1)
# cs_break=subset(cs_break, x > 0)
# plot(cs_break, log="xy")

cs_15break=count(15*round(sday_sess_break$V1/15))
cs_15break=subset(cs_15break, x > 0)
plot(cs_15break, log="xy", col=point_col,
	xlab="Separation duration (minutes)", ylab="Session pairs\n")

# Percentage interrupted sessions for each project
interrupt_perc=function(df)
{
return(100-length(which(df$time_log_interrupt_minutes == 0))/nrow(df)*100)
}

proj_int=ddply(timf, .(project_key), interrupt_perc)

mean(proj_int$V1)
sd(proj_int$V1)

plot(sort(proj_int$V1), col=point_col,
	xaxs="i",
	xlab="Anonymous project", ylab="Interrupted sessions (percent)\n")

# Elapsed time for task completed within same day

task_elapsed_time=function(df)
{
s_e=subset(df, start_time < end_time)
s_e$mins=(s_e$end_time-s_e$start_time)

c_mins=count(s_e$mins)
plot(c_mins, log="xy", col=pal_col[2],
	xlim=c(1, 600),
	xlab="Elapsed time (minutes)", ylab="Tasks\n")

x1=2:20
x2=21:240
dur_mod=glm(log(freq) ~ log(x), data=c_mins, subset=x1)
print(summary(dur_mod))
pred=predict(dur_mod)
lines(x1, exp(pred), col=pal_col[1])

dur_mod=glm(log(freq) ~ log(x), data=c_mins, subset=x2)
print(summary(dur_mod))
pred=predict(dur_mod, newdata=data.frame(x=x2))
lines(x2, exp(pred), col=pal_col[1])
}

task_start_time=function(start_time, col_str)
{
st15=count(start_time/60)
lines(st15$x, st15$freq/max(st15$freq, na.rm=TRUE), col=col_str)
}


same_day=subset(tf, start_date == end_date)

# space separates date from time
se_tf=subset(same_day, grepl(" ", task_actual_start_date) &
			grepl(" ", task_actual_complete_date))

se_tf$start_time=get_day_time(se_tf$task_actual_start_date)
se_tf$end_time=get_day_time(se_tf$task_actual_complete_date)

se_tf$start_15time=15*round(se_tf$start_time/15)

s_e=subset(se_tf, start_time < end_time)
s_e$mins=(s_e$end_time-s_e$start_time)

pal_col=rainbow(2)

plot(s_e$start_time/60, s_e$mins/60,
	xaxs="i", yaxs="i",
	xlab="Start time", ylab="Duration (hours)")


task_elapsed_time(subset(se_tf, project_key == 615))
task_elapsed_time(subset(se_tf, project_key == 614))

# Normalised number of tasks started at a given time of day

pal_col=rainbow(4)
plot(0, type="n",
	xaxs="i", yaxs="i",
	xlim=c(7, 23), ylim=c(0, 1),
	xlab="Time of day", ylab="Tasks (normalised)\n")

legend(x="topright", legend=c("614", "615", "617", "95"), bty="n", fill=pal_col, cex=1.3)

# Original time was mapped to MST, and need to map it to a common center
task_start_time(subset(se_tf, project_key == 615)$start_15time-1*60, pal_col[1])
task_start_time(subset(se_tf, project_key == 614)$start_15time-1*60, pal_col[2])
task_start_time(subset(se_tf, project_key == 617)$start_15time-1*60, pal_col[3])
task_start_time(subset(se_tf, project_key == 95)$start_15time+6*60, pal_col[4])


# Does estimation accuracy improve with practice?

# Add order id to each person's estimate
add_order_id=function(df)
{
# require a minimum number of estimates
if (nrow(df) < 10)
   return(NULL)
start_ord=order(df$start_date)
df$est_num[start_ord]=1:nrow(df)
return(df)
}

# Only consider construction phases.  Other phases appear to have a
# strong template calculation influence.
proj_con=subset(p617, phase_short_name %in% c_phase)

pord=ddply(proj_con, .(person_key), add_order_id)

act_mod=glm(log(task_actual_time_minutes) ~ log(task_plan_time_minutes)
					+factor(person_key)
					+log(est_num)
					# +(day_diff==0)
					# +log(num_sess)
					+phase_short_name
					# +(interrupt_minutes == 0)
					# +log(interrupt_minutes+1)
					, data=pord,
					subset=(task_plan_time_minutes > 0))
summary(act_mod)


# Spread of actual values for a given estimated value.

act_for_planned=function(plan_time, col_str=point_col)
{
h1=subset(tf, task_plan_time_minutes == plan_time)

ch1=count(h1$task_actual_time_minutes)

lines(ch1, col=col_str)
}

pal_col=rainbow(3)

plot(0.5, log="y",
	xlim=c(1, 180), ylim=c(1, 500),
	xlab="Actual time (minutes)", ylab="Tasks\n")

act_for_planned(30, pal_col[1])
act_for_planned(60, pal_col[2])
act_for_planned(120, pal_col[3])

legend(x="topright", legend=paste0("Estimated = ", c(30, 60, 120)), bty="n", fill=pal_col, cex=1.3)


# Spread of estimated values for a given actual value.

planned_for_act=function(actual_time, col_str=point_col)
{
h1=subset(tf, task_actual_time_minutes == actual_time)

ch1=count(round(h1$task_plan_time_minutes))

lines(ch1, col=col_str)
}

pal_col=rainbow(3)

plot(0.5, log="y",
	xlim=c(1, 180), ylim=c(1, 500),
	xlab="Estimated time (minutes)", ylab="Tasks\n")

planned_for_act(30, pal_col[1])
planned_for_act(60, pal_col[2])
planned_for_act(120, pal_col[3])

legend(x="topright", legend=paste0("Actual = ", c(30, 60, 120)), bty="n", fill=pal_col, cex=1.3)


# Build models based on task totals for each wbs

tf_wbs=ddply(tf, .(wbs_element_key), wbs_totals)
p615_wbs=ddply(p615, .(wbs_element_key), wbs_totals)

tf_wbs=subset(tf_wbs, est_total > 0)
p615_wbs=subset(p615_wbs, est_total > 0)

plot(tf_wbs$est_total, tf_wbs$act_total, log="xy", col=point_col)

wbs_mod=glm(log(act_total) ~ log(est_total)+num_persons+log(num_tasks), data=tf_wbs)
summary(wbs_mod)

wbs_dur=bizdays::bizdays(tf_wbs$start_date, tf_wbs$end_date, "US")
wbs_freq=count(wbs_dur)

plot(wbs_freq, log="xy",
	xlim=c(1, 500),
	xlab="Working days", ylab="WBS\n")

wbs_dur=bizdays::bizdays(p615_wbs$start_date, p615_wbs$end_date, "US")
wbs_freq=count(wbs_dur)

points(wbs_freq, col="red")


# Size facts

library("colorspace")

pal_col=rainbow(3)

par(las=1)
par(bty="l")
par(pch="+")

sf=read.csv("data/CESAW_size_facts.csv.xz", as.is=TRUE)

# t=which(sf$size_added_and_modified != (sf$size_added+sf$size_modified))

t=sf$size_added_and_modified - (sf$size_added+sf$size_modified)
tail(sort(table(t)))

sfc=subset(sf, size_metric_short_name == "LOC")
act_sfc=subset(sfc, measurement_type_name == "Actual")
plan_sfc=subset(sfc, measurement_type_name == "Plan")

LOC_plan_act=merge(plan_sfc, act_sfc, c("person_key", "plan_item_key"))

max_x=1e4
max_y=1e5

plot(LOC_plan_act$size_added_and_modified.x, LOC_plan_act$size_added_and_modified.y, log="xy", col=pal_col[3],
	xlim=c(1, max_x), ylim=c(1, max_y),
	xlab="Estimated LOC", ylab="Actual LOC")

legend(x="bottomright", legend=c("Fitted regression line", "Actual==Estimate"), bty="n", fill=pal_col[c(1, 2)], cex=1.3)


LOC_plan_act=subset(LOC_plan_act, size_added_and_modified.y > 0)
# lpa_mod=glm(log(size_added_and_modified.y) ~ log(size_added_and_modified.x)+
# 				             I(log(size_added_and_modified.x)^2), data=LOC_plan_act)
# lpa_mod=glm(log(size_added_and_modified.y) ~ log(size_added_and_modified.x),
# 					subset=(size_added_and_modified.x > 50),
# 							 data=LOC_plan_act)

lpa_mod=glm(log(size_added_and_modified.y) ~ log(size_added_and_modified.x),
							 data=LOC_plan_act)
summary(lpa_mod)

# Actual==Estimate
lines(c(1, max_y), c(1, max_y), col=pal_col[2])

x_range=1:max_x
pred=predict(lpa_mod, newdata=data.frame(size_added_and_modified.x=x_range))

lines(x_range, exp(pred), col=pal_col[1])

tsf=merge(tf, sf, by=c("project_key", "wbs_element_key"), all=TRUE)

tsf=merge(tf, sf, by=c("person_key", "plan_item_key"), all=TRUE)

# Get project_key and wbs_element_key mapping for tf * basic_... files

# bf=read.csv("data/basic_task_fact_sheet.csv", as.is=TRUE)

# Turns out that all the keys are different
# tbf=merge(tf, bf, by=c("person_key", "plan_item_key"), all=TRUE)

# The follows works a lot better, but there rows in both files that
# are not present in the other!
tbf=merge(tf, bf, by=c("task_actual_start_date", "task_plan_time_minutes"), all.x=TRUE)

pk_notNA=subset(tbf, (!is.na(project_key.x)) & !is.na(project_key.y))

pk_xy=data.frame(pk_notNA$project_key.x, pk_notNA$project_key.y)
t=pk_xy[!duplicated(pk_xy), ]

t[order(t$pk_notNA.project_key.x),]


